@@ -9,12 +9,6 @@
 /* Define if inline functions a la GCC are available.  */
 #undef HAVE_INLINE
 
-/* Define if sysinfo is available.  */
-#undef HAVE_SYSINFO
-
-/* Define if __NR_sysinfo is available.  */
-#undef HAVE_NRSYSINFO
-
 /* Define if inline functions a la GCC are available.  */
 #undef HAVE_ALPHASORT_DECLARATION
 
@@ -63,12 +57,11 @@
 /* Defined to the $(libdir) location */
 #undef LLIBDIR
 
-/* Is sysinfo() persent */
-#undef HAVE_SYSINFO
-
-/* Do we have proper memory information in struct sysinfo */
-#undef MEMINFO_IN_SYSINFO
-
 /* Define the ptrdiff_t type if your system doesn't have it */
 #undef ptrdiff_t
 
+/* Define if start-stop-daemon is compiled */
+#undef USE_START_STOP_DAEMON
+
+/* Define if you have the __va_copy macro */
+#undef HAVE_VA_COPY
