@@ -3,6 +3,7 @@
  * build.c - building archives
  *
  * Copyright (C) 1994,1995 Ian Jackson <iwj10@cus.cam.ac.uk>
+ * Copyright (C) 2000 Wichert Akkerman <wakkerma@debian.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as
@@ -33,6 +34,9 @@
 #include <limits.h>
 #include <ctype.h>
 #include <assert.h>
+#ifdef USE_ZLIB
+#include <zlib.h>
+#endif
 
 #include <config.h>
 #include <dpkg.h>
@@ -43,6 +47,18 @@
 # define S_ISLNK(mode) ((mode&0xF000) == S_IFLNK)
 #endif
 
+/* Simple structure to store information about a file.
+ */
+struct _finfo {
+  struct stat	st;
+  char*	fn;
+  struct _finfo* next;
+};
+
+/* Do a quick check if vstring is a valid versionnumber. Valid in this case
+ * means it contains at least one digit. If an error is found increment
+ * *errs.
+ */
 static void checkversion(const char *vstring, const char *valuename, int *errs) {
   const char *p;
   if (!vstring || !*vstring) return;
@@ -52,6 +68,137 @@ static void checkversion(const char *vstring, const char *valuename, int *errs)
   (*errs)++;
 }
 
+/* Read the next filename from a filedescriptor and create a _info struct
+ * for it. If there is nothing to read return NULL.
+ */
+static struct _finfo* getfi(const char* root, int fd) {
+  static char* fn = NULL;
+  static size_t fnlen = 0;
+  size_t i= 0;
+  struct _finfo *fi;
+  size_t rl = strlen(root);
+
+  if (fn == NULL) {
+    fnlen=rl+2048;
+    fn=(char*)malloc(fnlen);
+  } else if (fnlen < (rl+2048)) {
+    fnlen=rl+2048;
+    fn=(char*)realloc(fn,fnlen);
+  }
+  i=sprintf(fn,"%s/",root);
+  
+  while (1) {
+    int	res;
+    if (i>=fnlen) {
+      fnlen+=2048;
+      fn=(char*)realloc(fn,fnlen);
+    }
+    if ((res=read(fd, (fn+i), sizeof(*fn)))<0) {
+      if ((errno==EINTR) || (errno==EAGAIN))
+	continue;
+      else
+	return NULL;
+    }
+    if (res==0)	// EOF -> parent died
+      return NULL;
+    if (fn[i]==0)
+      break;
+
+    i++;
+    assert(i<2048);
+  }
+
+  fi=(struct _finfo*)malloc(sizeof(struct _finfo));
+  lstat(fn, &(fi->st));
+  fi->fn=strdup(fn+rl+1);
+  fi->next=NULL;
+  return fi;
+}
+
+/* Add a new _finfo struct to a single linked list of _finfo structs.
+ * We perform a slight optimization to work around a `feature' in tar: tar
+ * always recurses into subdirectories if you list a subdirectory. So if an
+ * entry is added and the previous entry in the list is its subdirectory we
+ * remove the subdirectory. 
+ *
+ * After a _finfo struct is added to a list it may no longer be freed, we
+ * assume full responsibility for its memory.
+ */
+static void add_to_filist(struct _finfo* fi, struct _finfo** start, struct _finfo **end) {
+  if (*start==NULL)
+    *start=*end=fi;
+  else 
+    *end=(*end)->next=fi;
+}
+
+/* Free the memory for all entries in a list of _finfo structs
+ */
+static void free_filist(struct _finfo* fi) {
+  while (fi) {
+    struct _finfo* fl;
+    free(fi->fn);
+    fl=fi; fi=fi->next;
+    free(fl);
+  }
+}
+
+int internalGzip(int fd1, int fd2, const char *compression, char *desc, ...) NONRETURNING;
+int internalGzip(int fd1, int fd2, const char *compression, char *desc, ...) {
+  va_list al;
+  struct varbuf v;
+#ifdef USE_ZLIB
+  gzFile gzfile;
+  char gzbuffer[4096];
+  int gzactualwrite, actualread;
+#endif
+  char combuf[6];
+
+  varbufinit(&v);
+
+  va_start(al,desc);
+  varbufvprintf(&v, desc, al);
+  va_end(al);
+
+  
+  if(compression == NULL) compression= "9";
+  if(*compression == '0') {
+    fd_fd_copy(0, 1, -1, _("%s: no compression copy loop"), v.buf);
+    exit(0);
+  }
+#ifdef USE_ZLIB
+  strncpy(combuf, "w9", sizeof(combuf));
+  combuf[1]= *compression;
+  gzfile = gzdopen(1, combuf);
+  while((actualread = read(0,gzbuffer,sizeof(gzbuffer))) > 0) {
+    if (actualread < 0 ) {
+      if (errno == EINTR) continue;
+      ohshite(_("%s: internal gzip error: read: `%s'"), v.buf, strerror(errno));
+    }
+    gzactualwrite= gzwrite(gzfile,gzbuffer,actualread);
+    if (gzactualwrite < 0 ) {
+      int gzerr = 0;
+      const char *errmsg = gzerror(gzfile, &gzerr);
+      if (gzerr == Z_ERRNO) {
+        if (errno == EINTR) continue;
+        errmsg= strerror(errno);
+      }
+      ohshite(_("%s: internal gzip error: write: `%s'"), v.buf, errmsg);
+    }
+    if (gzactualwrite != actualread)
+      ohshite(_("%s: internal gzip error: read(%i) != write(%i)"), v.buf, actualread, gzactualwrite);
+  }
+  gzclose(gzfile);
+  exit(0);
+#else
+  strncpy(combuf, "-9c", sizeof(combuf));
+  combuf[1]= *compression;
+  execlp(GZIP,"gzip",combuf,(char*)0); ohshit(_("%s: failed to exec gzip %s"), v.buf, combuf);
+#endif
+}
+
+/* Overly complex function that builds a .deb file
+ */
+void do_build(const char *const *argv) NONRETURNING;
 void do_build(const char *const *argv) {
   static const char *const maintainerscripts[]= {
     PREINSTFILE, POSTINSTFILE, PRERMFILE, POSTRMFILE, 0
@@ -59,20 +206,25 @@ void do_build(const char *const *argv) {
   
   char *m;
   const char *debar, *directory, *const *mscriptp, *versionstring, *arch;
-  char *controlfile, *tfbuf, *envbuf;
+  char *controlfile, *tfbuf;
+  const char *envbuf;
   struct pkginfo *checkedinfo;
   struct arbitraryfield *field;
   FILE *ar, *gz, *cf;
-  int p1[2],p2[2], warns, errs, n, c, subdir, gzfd;
-  pid_t c1,c2,c3,c4,c5;
+  int p1[2],p2[2],p3[2], warns, errs, n, c, subdir, gzfd;
+  pid_t c1,c2,c3;
   struct stat controlstab, datastab, mscriptstab, debarstab;
   char conffilename[MAXCONFFILENAME+1];
   time_t thetime= 0;
+  struct _finfo *fi;
+  struct _finfo *symlist = NULL;
+  struct _finfo *symlist_end = NULL;
   
+/* Decode our arguments */
   directory= *argv++; if (!directory) badusage(_("--build needs a directory argument"));
   /* template for our tempfiles */
   if ((envbuf= getenv("TMPDIR")) == NULL)
-    envbuf= (char *)P_tmpdir;
+    envbuf= P_tmpdir;
   tfbuf = (char *)malloc(strlen(envbuf)+13);
   strcpy(tfbuf,envbuf);
   strcat(tfbuf,"/dpkg.XXXXXX");
@@ -93,6 +245,8 @@ void do_build(const char *const *argv) {
     debar= m;
   }
     
+  /* Perform some sanity checks on the to-be-build package.
+   */
   if (nocheckflag) {
     if (subdir)
       ohshit(_("target is directory - cannot skip control file check"));
@@ -104,6 +258,7 @@ void do_build(const char *const *argv) {
                           sizeof(POSTINSTFILE) + sizeof(PREINSTFILE) +
                           sizeof(POSTRMFILE) + sizeof(PRERMFILE) +
                           MAXCONFFILENAME + 5);
+    /* Lets start by reading in the control-file so we can check its contents */
     strcpy(controlfile, directory);
     strcat(controlfile, "/" BUILDCONTROLDIR "/" CONTROLFILE);
     warns= 0; errs= 0;
@@ -139,6 +294,7 @@ void do_build(const char *const *argv) {
     }
     printf(_("dpkg-deb: building package `%s' in `%s'.\n"), checkedinfo->name, debar);
 
+    /* Check file permissions */
     strcpy(controlfile, directory);
     strcat(controlfile, "/" BUILDCONTROLDIR "/");
     if (lstat(controlfile,&mscriptstab)) ohshite("unable to stat control directory");
@@ -165,6 +321,7 @@ void do_build(const char *const *argv) {
       }
     }
 
+    /* Check if conffiles contains sane information */
     strcpy(controlfile, directory);
     strcat(controlfile, "/" BUILDCONTROLDIR "/" CONFFILESFILE);
     if ((cf= fopen(controlfile,"r"))) {
@@ -205,8 +362,12 @@ void do_build(const char *const *argv) {
   }
   if (ferror(stdout)) werr("stdout");
   
+  /* Now that we have verified everything its time to actually
+   * build something. Lets start by making the ar-wrapper.
+   */
   if (!(ar=fopen(debar,"wb"))) ohshite(_("unable to create `%.255s'"),debar);
   if (setvbuf(ar, 0, _IONBF, 0)) ohshite(_("unable to unbuffer `%.255s'"),debar);
+  /* Fork a tar to package the control-section of the package */
   m_pipe(p1);
   if (!(c1= m_fork())) {
     m_dup2(p1[1],1); close(p1[0]); close(p1[1]);
@@ -215,6 +376,9 @@ void do_build(const char *const *argv) {
     execlp(TAR,"tar","-cf","-",".",(char*)0); ohshite(_("failed to exec tar -cf"));
   }
   close(p1[1]);
+  /* Create a temporary file to store the control data in. Immediately unlink
+   * our temporary file so others can't mess with it.
+   */
   if ((gzfd= mkstemp(tfbuf)) == -1) ohshite(_("failed to make tmpfile (control)"));
   if ((gz= fdopen(gzfd,"a")) == NULL) ohshite(_("failed to open tmpfile "
       "(control), %s"), tfbuf);
@@ -224,14 +388,18 @@ void do_build(const char *const *argv) {
   /* reset this, so we can use it elsewhere */
   strcpy(tfbuf,envbuf);
   strcat(tfbuf,"/dpkg.XXXXXX");
+  /* And run gzip to compress our control archive */
   if (!(c2= m_fork())) {
     m_dup2(p1[0],0); m_dup2(gzfd,1); close(p1[0]); close(gzfd);
-    execlp(GZIP,"gzip","-9c",(char*)0); ohshite(_("failed to exec gzip -9c"));
+    internalGzip(0, 1, "9", _("control"));
   }
   close(p1[0]);
   waitsubproc(c2,"gzip -9c",0);
   waitsubproc(c1,"tar -cf",0);
   if (fstat(gzfd,&controlstab)) ohshite(_("failed to fstat tmpfile (control)"));
+  /* We have our first file for the ar-archive. Write a header for it to the
+   * package and insert it.
+   */
   if (oldformatflag) {
     if (fprintf(ar, "%-8s\n%ld\n", OLDARCHIVEVERSION, (long)controlstab.st_size) == EOF)
       werr(debar);
@@ -252,8 +420,11 @@ void do_build(const char *const *argv) {
   }                
                 
   if (lseek(gzfd,0,SEEK_SET)) ohshite(_("failed to rewind tmpfile (control)"));
-  do_fd_copy(gzfd, fileno(ar), _("control"));
+  fd_fd_copy(gzfd, fileno(ar), -1, _("control"));
 
+  /* Control is done, now we need to archive the data. Start by creating
+   * a new temporary file. Immediately unlink the temporary file so others
+   * can't mess with it. */
   if (!oldformatflag) {
     fclose(gz);
     if ((gzfd= mkstemp(tfbuf)) == -1) ohshite(_("failed to make tmpfile (data)"));
@@ -266,32 +437,61 @@ void do_build(const char *const *argv) {
     strcpy(tfbuf,envbuf);
     strcat(tfbuf,"/dpkg.XXXXXX");
   }
+  /* Fork off a tar. We will feed it a list of filenames on stdin later.
+   */
+  m_pipe(p1);
   m_pipe(p2);
-  if (!(c4= m_fork())) {
+  if (!(c1= m_fork())) {
+    m_dup2(p1[0],0); close(p1[0]); close(p1[1]);
     m_dup2(p2[1],1); close(p2[0]); close(p2[1]);
     if (chdir(directory)) ohshite(_("failed to chdir to `%.255s'"),directory);
-    execlp(TAR,"tar","--exclude",BUILDCONTROLDIR,"-cf","-",".",(char*)0);
-    ohshite(_("failed to exec tar --exclude"));
+    execlp(TAR,"tar","-cf", "-", "-T", "-", "--null", "--no-recursion", (char*)0);
+    ohshite(_("failed to exec tar -cf"));
   }
+  close(p1[0]);
   close(p2[1]);
-  if (!(c5= m_fork())) {
-    char *combuf;
+  /* Of course we should not forget to compress the archive as well.. */
+  if (!(c2= m_fork())) {
+    close(p1[1]);
     m_dup2(p2[0],0); close(p2[0]);
     m_dup2(oldformatflag ? fileno(ar) : gzfd,1);
-    combuf = strdup("-9c");
-    if(compression != NULL) {
-      if(*compression == '0') {
-	do_fd_copy(0, 1, _("no compression copy loop"));
-	exit(0);
-      }
-      combuf[1] = *compression;
-    }
-    execlp(GZIP,"gzip",combuf,(char*)0);
-    ohshite(_("failed to exec gzip %s from tar --exclude"), combuf);
+    internalGzip(0, 1, compression, _("control"));
   }
   close(p2[0]);
-  waitsubproc(c5,"gzip -9c from tar --exclude",0);
-  waitsubproc(c4,"tar --exclude",0);
+  /* All the pipes are set, now lets run find, and start feeding
+   * filenames to tar.
+   */
+
+  m_pipe(p3);
+  if (!(c3= m_fork())) {
+    m_dup2(p3[1],1); close(p3[0]); close(p3[1]);
+    if (chdir(directory)) ohshite(_("failed to chdir to `%.255s'"),directory);
+    execlp(FIND,"find",".","-path","./" BUILDCONTROLDIR,"-prune","-o","-print0",(char*)0);
+    ohshite(_("failed to exec find"));
+  }
+  close(p3[1]);
+  /* We need to reorder the files so we can make sure that symlinks
+   * will not appear before their target.
+   */
+  while ((fi=getfi(directory, p3[0]))!=NULL)
+    if (S_ISLNK(fi->st.st_mode))
+      add_to_filist(fi,&symlist,&symlist_end);
+    else {
+      if (write(p1[1], fi->fn, strlen(fi->fn)+1) ==- 1)
+	ohshite(_("failed to write filename to tar pipe (data)"));
+    }
+  close(p3[0]);
+  waitsubproc(c3,"find",0);
+
+  for (fi= symlist;fi;fi= fi->next)
+    if (write(p1[1], fi->fn, strlen(fi->fn)+1) == -1)
+      ohshite(_("failed to write filename to tar pipe (data)"));
+  /* All done, clean up wait for tar and gzip to finish their job */
+  close(p1[1]);
+  free_filist(symlist);
+  waitsubproc(c2,"gzip -9c from tar -cf",0);
+  waitsubproc(c1,"tar -cf",0);
+  /* Okay, we have data.tar.gz as well now, add it to the ar wrapper */
   if (!oldformatflag) {
     if (fstat(gzfd,&datastab)) ohshite("_(failed to fstat tmpfile (data))");
     if (fprintf(ar,
@@ -303,11 +503,7 @@ void do_build(const char *const *argv) {
       werr(debar);
 
     if (lseek(gzfd,0,SEEK_SET)) ohshite(_("failed to rewind tmpfile (data)"));
-    if (!(c3= m_fork())) {
-      m_dup2(gzfd,0); m_dup2(fileno(ar),1);
-      execlp(CAT,"cat",(char*)0); ohshite(_("failed to exec cat (data)"));
-    }
-    waitsubproc(c3,"cat (data)",0);
+    fd_fd_copy(gzfd, fileno(ar), -1, _("cat (data)"));
 
     if (datastab.st_size & 1)
       if (putc('\n',ar) == EOF)
