@@ -33,6 +33,9 @@
 #include <ctype.h>
 #include <assert.h>
 #include <ar.h>
+#ifdef USE_ZLIB
+#include <zlib.h>
+#endif
 
 #include <config.h>
 #include <dpkg.h>
@@ -50,6 +53,7 @@ static void movecontrolfiles(const char *thing) {
   waitsubproc(c1,"sh -c mv foo/* &c",0);
 }
 
+static void readfail(FILE *a, const char *filename, const char *what) NONRETURNING;
 static void readfail(FILE *a, const char *filename, const char *what) {
   if (ferror(a)) {
     ohshite(_("error reading %s from %.255s"),what,filename);
@@ -76,22 +80,12 @@ static unsigned long parseheaderlength(const char *inh, size_t len,
   return r;
 }
 
-static void skipmember(FILE *ar, const char *fn, long memberlen) {
-  int c;
-  
-  memberlen += (memberlen&1);
-  while (memberlen > 0) {
-    if ((c= getc(ar)) == EOF) readfail(ar,fn,"skipped member data");
-    memberlen--;
-  }
-}
-
 void extracthalf(const char *debar, const char *directory,
                  const char *taroption, int admininfo) {
   char versionbuf[40];
   float versionnum;
   char ctrllenbuf[40], *infobuf;
-  long ctrllennum, memberlen= 0;
+  size_t ctrllennum, memberlen= 0;
   int dummy, l= 0;
   pid_t c1=0,c2,c3;
   unsigned char *ctrlarea= 0;
@@ -101,14 +95,19 @@ void extracthalf(const char *debar, const char *directory,
   char nlc;
   char *cur;
   struct ar_hdr arh;
-  int readfromfd, oldformat, header_done, adminmember, c;
+  int readfromfd, oldformat= 0, header_done, adminmember;
 #if defined(__GLIBC__) && (__GLIBC__ == 2) && (__GLIBC_MINOR__ > 0)
   fpos_t fpos;
 #endif
+#ifdef USE_ZLIB
+  gzFile gzfile;
+  char gzbuffer[4096];
+  int gzactualread;
+#endif
   
   ar= fopen(debar,"r"); if (!ar) ohshite(_("failed to read archive `%.255s'"),debar);
   if (fstat(fileno(ar),&stab)) ohshite(_("failed to fstat archive"));
-  if (!fgets(versionbuf,sizeof(versionbuf),ar)) readfail(ar,debar,"version number");
+  if (!fgets(versionbuf,sizeof(versionbuf),ar)) readfail(ar,debar,_("version number"));
 
   if (!strcmp(versionbuf,"!<arch>\n")) {
     oldformat= 0;
@@ -117,20 +116,20 @@ void extracthalf(const char *debar, const char *directory,
     header_done= 0;
     for (;;) {
       if (fread(&arh,1,sizeof(arh),ar) != sizeof(arh))
-        readfail(ar,debar,"between members");
+        readfail(ar,debar,_("between members"));
       if (memcmp(arh.ar_fmag,ARFMAG,sizeof(arh.ar_fmag)))
         ohshit(_("file `%.250s' is corrupt - bad magic at end of first header"),debar);
       memberlen= parseheaderlength(arh.ar_size,sizeof(arh.ar_size),
                                    debar,"member length");
       if (memberlen<0)
-        ohshit(_("file `%.250s' is corrupt - negative member length %ld"),debar,memberlen);
+        ohshit(_("file `%.250s' is corrupt - negative member length %zi"),debar,memberlen);
       if (!header_done) {
         if (memcmp(arh.ar_name,"debian-binary   ",sizeof(arh.ar_name)) &&
 	    memcmp(arh.ar_name,"debian-binary/   ",sizeof(arh.ar_name)))
           ohshit(_("file `%.250s' is not a debian binary archive (try dpkg-split?)"),debar);
         infobuf= m_malloc(memberlen+1);
         if (fread(infobuf,1, memberlen + (memberlen&1), ar) != memberlen + (memberlen&1))
-          readfail(ar,debar,"header info member");
+          readfail(ar,debar,_("header info member"));
         infobuf[memberlen]= 0;
         cur= strchr(infobuf,'\n');
         if (!cur) ohshit(_("archive has no newlines in header"));
@@ -148,7 +147,7 @@ void extracthalf(const char *debar, const char *directory,
           /* Members with `_' are noncritical, and if we don't understand them
            * we skip them.
            */
-        skipmember(ar,debar,memberlen);
+	stream_null_copy(ar, memberlen + (memberlen&1),_("skipped member data from %s"), debar);
       } else {
         adminmember=
           (!memcmp(arh.ar_name,ADMINMEMBER,sizeof(arh.ar_name)) ||
@@ -166,7 +165,7 @@ void extracthalf(const char *debar, const char *directory,
           ctrllennum= memberlen;
         }
         if (!adminmember != !admininfo) {
-          skipmember(ar,debar,memberlen);
+	  stream_null_copy(ar, memberlen + (memberlen&1),_("skipped member data from %s"), debar);
         } else {
           break; /* Yes ! - found it. */
         }
@@ -175,7 +174,7 @@ void extracthalf(const char *debar, const char *directory,
 
     if (admininfo >= 2)
       if (printf(_(" new debian package, version %s.\n"
-                 " size %ld bytes: control archive= %ld bytes.\n"),
+                 " size %ld bytes: control archive= %zi bytes.\n"),
                  versionbuf, (long)stab.st_size, ctrllennum) == EOF ||
           fflush(stdout)) werr("stdout");
     
@@ -186,13 +185,13 @@ void extracthalf(const char *debar, const char *directory,
     oldformat= 1;
     l= strlen(versionbuf); if (l && versionbuf[l-1]=='\n') versionbuf[l-1]=0;
     if (!fgets(ctrllenbuf,sizeof(ctrllenbuf),ar))
-      readfail(ar,debar,"ctrl information length");
-    if (sscanf(ctrllenbuf,"%ld%c%d",&ctrllennum,&nlc,&dummy) !=2 || nlc != '\n')
+      readfail(ar,debar,_("ctrl information length"));
+    if (sscanf(ctrllenbuf,"%zi%c%d",&ctrllennum,&nlc,&dummy) !=2 || nlc != '\n')
       ohshit(_("archive has malformatted ctrl len `%s'"),ctrllenbuf);
 
     if (admininfo >= 2)
       if (printf(_(" old debian package, version %s.\n"
-                 " size %ld bytes: control archive= %ld, main archive= %ld.\n"),
+                 " size %ld bytes: control archive= %ld, main archive= %zi.\n"),
                  versionbuf, (long)stab.st_size, ctrllennum,
                  (long) (stab.st_size - ctrllennum - strlen(ctrllenbuf) - l)) == EOF ||
           fflush(stdout)) werr("stdout");
@@ -200,7 +199,7 @@ void extracthalf(const char *debar, const char *directory,
     ctrlarea= malloc(ctrllennum); if (!ctrlarea) ohshite("malloc ctrlarea failed");
 
     errno=0; if (fread(ctrlarea,1,ctrllennum,ar) != ctrllennum)
-      readfail(ar,debar,"ctrlarea");
+      readfail(ar,debar,_("ctrlarea"));
 
   } else {
     
@@ -245,13 +244,8 @@ void extracthalf(const char *debar, const char *directory,
     m_pipe(p1);
     if (!(c1= m_fork())) {
       close(p1[0]);
-      if (!(pi= fdopen(p1[1],"w"))) ohshite(_("failed to fdopen p1 in copy"));
-      while (memberlen > 0) {
-        if ((c= getc(ar)) == EOF) readfail(ar,debar,"member data");
-        if (putc(c,pi) == EOF) ohshite(_("failed to write to pipe in copy"));
-        memberlen--;
-      }
-      if (fclose(pi) == EOF) ohshite(_("failed to close pipe in copy"));
+      stream_fd_copy(ar, p1[1], memberlen, _("failed to write to pipe in copy"));
+      if (close(p1[1]) == EOF) ohshite(_("failed to close pipe in copy"));
       exit(0);
     }
     close(p1[1]);
@@ -264,7 +258,24 @@ void extracthalf(const char *debar, const char *directory,
     m_dup2(readfromfd,0);
     if (admininfo) close(p1[0]);
     if (taroption) { m_dup2(p2[1],1); close(p2[0]); close(p2[1]); }
+#ifdef USE_ZLIB
+    gzfile = gzdopen(0, "r");
+    while ((gzactualread= gzread(gzfile,gzbuffer,sizeof(gzbuffer))) > 0) {
+      if (gzactualread < 0 ) {
+	int gzerr = 0;
+	const char *errmsg = gzerror(gzfile, &gzerr);
+	if (gzerr == Z_ERRNO) {
+	  if (errno == EINTR) continue;
+	  errmsg= strerror(errno);
+	}
+	ohshite(_("internal gzip error: `%s'"), errmsg);
+      }
+      write(1,gzbuffer,gzactualread);
+    }
+    exit(0);
+#else
     execlp(GZIP,"gzip","-dc",(char*)0); ohshite(_("failed to exec gzip -dc"));
+#endif
   }
   if (readfromfd != fileno(ar)) close(readfromfd);
   if (taroption) close(p2[1]);
@@ -295,7 +306,7 @@ void extracthalf(const char *debar, const char *directory,
     waitsubproc(c3,"tar",0);
   }
   
-  waitsubproc(c2,"gzip -dc",1);
+  waitsubproc(c2,"gzip -dc",PROCPIPE);
   if (c1 != -1) waitsubproc(c1,"paste",0);
   if (oldformat && admininfo) {
     if (versionnum == 0.931F) {
