@@ -25,6 +25,10 @@
 #define TOTAL_LIST_WIDTH 180
 #define MAX_DISPLAY_INFO 120
 
+#include <signal.h>
+#undef ERR
+#include <curses.h>
+
 struct helpmenuentry {
   char key;
   const struct helpmessage *msg;
@@ -136,6 +140,7 @@ void cursesoff();
 
 extern const char *admindir;
 extern FILE *debug;
+extern int expertmode;
 
 enum urqresult { urqr_normal, urqr_fail, urqr_quitmenu };
 enum quitaction { qa_noquit, qa_quitchecksave, qa_quitnochecksave };
