@@ -1,8 +1,9 @@
 /*
- * dselect - Debian GNU/Linux package maintenance user interface
+ * dselect - Debian package maintenance user interface
  * main.cc - main program
  *
  * Copyright (C) 1994,1995 Ian Jackson <iwj10@cus.cam.ac.uk>
+ * Copyright (C) 2000 Wichert Akkerman <wakkerma@debian.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as
@@ -60,30 +61,35 @@ int expertmode = 0;
 static keybindings packagelistbindings(packagelist_kinterps,packagelist_korgbindings);
 
 struct menuentry {
+  const char *command;
+  const char *key;
   const char *option;
   const char *menuent;
   urqfunction *fn;
 };
 
 static const menuentry menuentries[]= {
-  { N_("access"),  N_("Choose the access method to use."),                &urq_setup   },
-  { N_("update"),  N_("Update list of available packages, if possible."), &urq_update  },
-  { N_("select"),  N_("Request which packages you want on your system."), &urq_list    },
-  { N_("install"), N_("Install and upgrade wanted packages."),            &urq_install },
-  { N_("config"),  N_("Configure any packages that are unconfigured."),   &urq_config  },
-  { N_("remove"),  N_("Remove unwanted software."),                       &urq_remove  },
-  { N_("quit"),    N_("Quit dselect."),                                   &urq_quit    },
-  { N_("menu"),    0,                                                     &urq_menu    },
-  { 0                                                                              }
+  { "access",	N_("a"),	N_("[A]ccess"),	N_("Choose the access method to use."),			&urq_setup   },
+  { "update",	N_("u"),	N_("[U]pdate"),	N_("Update list of available packages, if possible."),	&urq_update  },
+  { "select",	N_("s"),	N_("[S]elect"),	N_("Request which packages you want on your system."),	&urq_list    },
+  { "install",	N_("i"),	N_("[I]nstall"),N_("Install and upgrade wanted packages."),		&urq_install },
+  { "config",	N_("c"),	N_("[C]onfig"),	N_("Configure any packages that are unconfigured."),	&urq_config  },
+  { "remove",	N_("r"),	N_("[R]emove"),	N_("Remove unwanted software."),			&urq_remove  },
+  { "quit",	N_("q"),	N_("[Q]uit"),	N_("Quit dselect."),					&urq_quit    },
+  { 0,		0,  		N_("menu"),	0,							&urq_menu    },
+  { 0 }
 };
 
 static const char programdesc[]=
-      N_("Debian GNU/Linux `%s' package handling frontend.");
+      N_("Debian `%s' package handling frontend.");
 
 static const char copyrightstring[]= N_(
-      "Version %s.  Copyright (C) 1994-1996 Ian Jackson.   This is\n"
-      "free software; see the GNU General Public Licence version 2 or later for\n"
-      "copying conditions.  There is NO warranty.  See dselect --licence for details.\n");
+      "Version %s.\n"
+      "Copyright (C) 1994-1996 Ian Jackson.\n"
+      "Copyright (C) 2000 Wichert Akkerman.\n"
+      "This is free software; see the GNU General Public Licence version 2\n"
+      "or later for copying conditions.  There is NO warranty.  See\n"
+      "dselect --licence for details.\n");
 
 static void printversion(void) {
   if (fprintf(stdout,gettext(programdesc),DSELECT) == EOF) werr("stdout");
@@ -117,7 +123,7 @@ extern "C" {
     setvbuf(debug,0,_IONBF,0);
   }
 
-  static void setexpert() {
+  static void setexpert(const struct cmdinfo*, const char *v) {
     expertmode = 1;
   }
 
@@ -131,7 +137,7 @@ static const struct cmdinfo cmdinfos[]= {
   { "version",    0,   0,  0,  0,          versiononly            },
   { "licence",    0,   0,  0,  0,          showcopyright          }, /* UK spelling */
   { "license",    0,   0,  0,  0,          showcopyright          }, /* US spelling */
-  {  0,           0                                               }
+  {  0,           0,   0,  0,  0,          0                      }
 };
 
 static int cursesareon= 0;
@@ -194,10 +200,9 @@ urqresult urq_list(void) {
 void dme(int i, int so) {
   char buf[120];
   const menuentry *me= &menuentries[i];
-  const char* option = gettext(me->option);
-  sprintf(buf," %c %d. [%c]%-10.10s %-80.80s ",
+  sprintf(buf," %c %d. %-11.11s %-80.80s ",
           so ? '*' : ' ', i,
-          toupper(option[0]), option+1,
+          gettext(me->option),
           gettext(me->menuent));
   
   int y,x;
@@ -257,11 +262,22 @@ urqresult urq_menu(void) {
   dme(0,1);
   for (;;) {
     refresh();
-    c= getch();  if (c==ERR) ohshite(_("failed to getch in main menu"));
-    if (c==C('n') || c==KEY_DOWN || c==' ') {
+    do
+      c= getch();
+    while (c == ERR && errno == EINTR);
+    if (c==ERR)  {
+      if(errno != 0)
+        ohshite(_("failed to getch in main menu"));
+      else {
+        clearok(stdscr,TRUE); clear(); refreshmenu(); dme(cursor,1); 
+      }
+    }
+
+    if (c==ERR) ohshite(_("failed to getch in main menu"));
+    if (c==C('n') || c==KEY_DOWN || c==' ' || c=='j') {
       dme(cursor,0); cursor++; cursor %= entries; dme(cursor,1);
     } else if (c==C('p') || c==KEY_UP || c==C('h') ||
-               c==KEY_BACKSPACE || c==KEY_DC) {
+               c==KEY_BACKSPACE || c==KEY_DC || c=='k') {
       dme(cursor,0); cursor+= entries-1; cursor %= entries; dme(cursor,1);
     } else if (c=='\n' || c=='\r' || c==KEY_ENTER) {
       clear(); refresh();
@@ -287,7 +303,7 @@ urqresult urq_menu(void) {
       }
     } else if (isalpha(c)) {
       c= tolower(c);
-      for (i=0; i<entries && gettext(menuentries[i].option)[0] != c; i++);
+      for (i=0; i<entries && gettext(menuentries[i].key)[0] != c; i++);
       if (i < entries) {
         dme(cursor,0); cursor=i; dme(cursor,1);
       } else {
@@ -317,14 +333,15 @@ int main(int, const char *const *argv) {
   }
   push_error_handler(&ejbuf,print_error_fatal,0);
 
+  myfileopt(CONFIGDIR "/" DSELECT ".cfg", cmdinfos);
   myopt(&argv,cmdinfos);
 
   if (*argv) {
     const char *a;
     while ((a= *argv++) != 0) {
       const menuentry *me;
-      for (me= menuentries; me->option && strcmp(me->option,a); me++);
-      if (!me->option) badusage(_("unknown action string `%.50s'"),a);
+      for (me= menuentries; me->command && strcmp(me->command,a); me++);
+      if (!me->command) badusage(_("unknown action string `%.50s'"),a);
       me->fn();
     }
   } else {
