@@ -119,9 +119,9 @@ methodlist::methodlist() : baselist(&methodlistbindings) {
   if (debug)
     fprintf(debug,"methodlist[%p]::methodlist()\n",this);
 
-  table= new struct option*[noptions];
+  table= new struct dselect_option*[noptions];
 
-  struct option *opt, **ip;
+  struct dselect_option *opt, **ip;
   for (opt=options, ip=table, nitems=0; opt; opt=opt->next, nitems++) {
     if (opt == coption) { assert(newcursor==-1); newcursor= nitems; }
     *ip++= opt;
@@ -156,7 +156,7 @@ quitaction methodlist::display() {
     signallist= this;
     if (sigprocmask(SIG_UNBLOCK,&sigwinchset,0)) ohshite(_("failed to unblock SIGWINCH"));
     do
-      response= getch();
+    response= getch();
     while (response == ERR && errno == EINTR);
     if (sigprocmask(SIG_BLOCK,&sigwinchset,0)) ohshite(_("failed to re-block SIGWINCH"));
     if (response == ERR) ohshite(_("getch failed"));
