@@ -21,9 +21,7 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <curses.h>
 #include <assert.h>
-#include <signal.h>
 
 extern "C" {
 #include <config.h>
