@@ -77,7 +77,6 @@ const keybindings::interpretation packagelist_kinterps[] = {
 
 const keybindings::orgbinding packagelist_korgbindings[]= {
   { 'j',            "down"             }, // vi style
-  { 'n',            "down"             },
   { KEY_DOWN,       "down"             },
   { 'k',            "up"               }, // vi style
   { 'p',            "up"               },
@@ -133,6 +132,7 @@ const keybindings::orgbinding packagelist_korgbindings[]= {
   { C('l'),         "redraw"           },
   { '/',            "search"           },
   { '\\',           "searchagain"      },
+  { 'n',            "searchagain"      },
                                        
   { KEY_ENTER,      "quitcheck"        },
   { '\r',           "quitcheck"        },
