@@ -183,7 +183,7 @@ void packagelist::redraw1itemsel(int index, int selected) {
       wmove(listpad,index,priority_column-1); waddch(listpad,' ');
       if (pkg->priority == pkginfo::pri_other) {
         int i;
-        char *p;
+        const char *p;
         for (i=priority_width, p=pkg->otherpriority;
              i > 0 && *p;
              i--, p++)
