@@ -24,6 +24,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
 struct versionrevision {
   unsigned long epoch;
@@ -80,8 +81,8 @@ struct arbitraryfield {
 
 struct conffile {
   struct conffile *next;
-  char *name;
-  char *hash;
+  const char *name;
+  const char *hash;
 };
 
 struct filedetails {
@@ -97,17 +98,17 @@ struct pkginfoperfile { /* pif */
   struct dependency *depends;
   struct deppossi *depended;
   int essential; /* The `essential' flag, 1=yes, 0=no (absent) */
-  char *description, *maintainer, *source, *architecture, *installedsize;
+  char *description, *maintainer, *source, *architecture, *installedsize, *origin, *bugs;
   struct versionrevision version;
   struct conffile *conffiles;
   struct arbitraryfield *arbs;
 };
 
-struct perpackagestate; /* used by dselect only, but we keep a pointer here */
+struct perpackagestate; /* dselect and dpkg have different versions of this */
 
 struct pkginfo { /* pig */
   struct pkginfo *next;
-  char *name;
+  const char *name;
   enum pkgwant {
     want_unknown, want_install, want_hold, want_deinstall, want_purge,
     want_sentinel /* Not allowed except as special sentinel value
@@ -130,8 +131,8 @@ struct pkginfo { /* pig */
     pri_optional, pri_extra, pri_contrib,
     pri_other, pri_unknown, pri_unset=-1
   } priority;
-  char *otherpriority;
-  char *section;
+  const char *otherpriority;
+  const char *section;
   struct versionrevision configversion;
   struct filedetails *files;
   struct pkginfoperfile installed;
@@ -157,6 +158,12 @@ enum modstatdb_rw {
   msdbrw_noavail= 0100,
 };
 
+struct pipef {
+  int fd;
+  struct pipef *next;
+};
+extern struct pipef *status_pipes;
+
 enum modstatdb_rw modstatdb_init(const char *admindir, enum modstatdb_rw reqrwflags);
 void modstatdb_note(struct pkginfo *pkg);
 void modstatdb_shutdown(void);
@@ -200,7 +207,7 @@ void copy_dependency_links(struct pkginfo *pkg,
 
 struct namevalue {
   const char *name;
-  int value;
+  int value, length;
 };
 
 extern const struct namevalue booleaninfos[];
@@ -225,11 +232,15 @@ const char *versiondescribe(const struct versionrevision*,
 struct varbuf;
 
 extern void varbufaddc(struct varbuf *v, int c);
+extern void varbufdupc(struct varbuf *v, int c, ssize_t s);
+int varbufprintf(struct varbuf *v, const char *fmt, ...) PRINTFFORMAT(2,3);
+int varbufvprintf(struct varbuf *v, const char *fmt, va_list va);
 void varbufinit(struct varbuf *v);
 void varbufreset(struct varbuf *v);
 void varbufextend(struct varbuf *v);
 void varbuffree(struct varbuf *v);
-void varbufaddstr(struct varbuf *v, const char *s);
+#define varbufaddstr(v, s)      varbufaddbuf(v, s, strlen(s))
+extern void varbufaddbuf(struct varbuf *v, const void *s, const int l);
 
 /* varbufinit must be called exactly once before the use of each varbuf
  * (including before any call to varbuffree).
@@ -246,7 +257,7 @@ void varbufaddstr(struct varbuf *v, const char *s);
  * Callers using C++ need not worry about any of this.
  */
 struct varbuf {
-  int used, size;
+  size_t used, size;
   char *buf;
 
 #ifdef __cplusplus
@@ -254,26 +265,14 @@ struct varbuf {
   void free() { varbuffree(this); }
   varbuf() { varbufinit(this); }
   ~varbuf() { varbuffree(this); }
-  inline void operator()(int c); // definition below
+  void operator()(int c) { varbufaddc(this,c); }
   void operator()(const char *s) { varbufaddstr(this,s); }
-  inline void terminate(void/*to shut 2.6.3 up*/); // definition below
+  void terminate(void/*to shut 2.6.3 up*/) { varbufaddc(this,0); used--; }
   void reset() { used=0; }
   const char *string() { terminate(); return buf; }
 #endif
 };
 
-#if HAVE_INLINE
-inline extern void varbufaddc(struct varbuf *v, int c) {
-  if (v->used >= v->size) varbufextend(v);
-  v->buf[v->used++]= c;
-}
-#endif
-
-#ifdef __cplusplus
-inline void varbuf::operator()(int c) { varbufaddc(this,c); }
-inline void varbuf::terminate(void/*to shut 2.6.3 up*/) { varbufaddc(this,0); used--; }
-#endif
-
 /*** from dump.c ***/
 
 void writerecord(FILE*, const char*,
@@ -283,7 +282,6 @@ void writedb(const char *filename, int available, int mustsync);
 
 void varbufrecord(struct varbuf*, const struct pkginfo*, const struct pkginfoperfile*);
 void varbufdependency(struct varbuf *vb, struct dependency *dep);
-void varbufprintf(struct varbuf *v, const char *fmt, ...) PRINTFFORMAT(2,3);
   /* NB THE VARBUF MUST HAVE BEEN INITIALISED AND WILL NOT BE NULL-TERMINATED */
 
 /*** from vercmp.c ***/
@@ -298,9 +296,9 @@ int epochsdiffer(const struct versionrevision *a,
                  const struct versionrevision *b);
 
 /*** from nfmalloc.c ***/
-
-void *nfmalloc(size_t);
+extern void *nfmalloc(size_t);
 char *nfstrsave(const char*);
+char *nfstrnsave(const char*, int);
 void nffreeall(void);
 
 #endif /* DPKG_DB_H */
