@@ -3,6 +3,7 @@
  * dbmodify.c - routines for managing dpkg database updates
  *
  * Copyright (C) 1994,1995 Ian Jackson <iwj10@cus.cam.ac.uk>
+ * Copyright (C) 2001 Wichert Akkerman <wichert@debian.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as
@@ -37,10 +38,10 @@
 #include <dpkg.h>
 #include <dpkg-db.h>
 
-char *statusfile=0, *availablefile=0;
+char *statusfile=NULL, *availablefile=NULL;
 
 static enum modstatdb_rw cstatus=-1, cflags=0;
-static char *importanttmpfile=0;
+static char *importanttmpfile=NULL;
 static FILE *importanttmp;
 static int nextupdate;
 static int updateslength;
@@ -67,7 +68,7 @@ static void cleanupdates(void) {
   struct dirent **cdlist;
   int cdn, i;
 
-  parsedb(statusfile, pdb_weakclassification, 0,0,0);
+  parsedb(statusfile, pdb_weakclassification, NULL,NULL,NULL);
 
   *updatefnrest= 0;
   updateslength= -1;
@@ -78,7 +79,7 @@ static void cleanupdates(void) {
     
     for (i=0; i<cdn; i++) {
       strcpy(updatefnrest, cdlist[i]->d_name);
-      parsedb(updatefnbuf, pdb_weakclassification, 0,0,0);
+      parsedb(updatefnbuf, pdb_weakclassification, NULL,NULL,NULL);
       if (cstatus < msdbrw_write) free(cdlist[i]);
     }
 
@@ -100,12 +101,13 @@ static void cleanupdates(void) {
 }
 
 static void createimptmp(void) {
-  int i;
+  int i, f;
   
   onerr_abort++;
   
   importanttmp= fopen(importanttmpfile,"w");
   if (!importanttmp) ohshite(_("unable to create %.250s"),importanttmpfile);
+  setcloexec(fileno(importanttmp),importanttmpfile);
   for (i=0; i<512; i++) fputs("#padding\n",importanttmp);
   if (ferror(importanttmp))
     ohshite(_("unable to fill %.250s with padding"),importanttmpfile);
@@ -122,11 +124,11 @@ enum modstatdb_rw modstatdb_init(const char *adir, enum modstatdb_rw readwritere
     {   STATUSFILE,                 &statusfile         },
     {   AVAILFILE,                  &availablefile      },
     {   UPDATESDIR IMPORTANTTMP,    &importanttmpfile   },
-    {   0                                               }
+    {   NULL, NULL                                      }
   }, *fnip;
-
-  admindir= adir;
   
+  admindir= adir;
+
   for (fnip=fnis; fnip->suffix; fnip++) {
     free(*fnip->store);
     *fnip->store= m_malloc(strlen(adir)+strlen(fnip->suffix)+2);
@@ -172,7 +174,7 @@ enum modstatdb_rw modstatdb_init(const char *adir, enum modstatdb_rw readwritere
     if(!(cflags & msdbrw_noavail))
     parsedb(availablefile,
             pdb_recordavailable|pdb_rejectstatus,
-            0,0,0);
+            NULL,NULL,NULL);
   }
 
   if (cstatus >= msdbrw_write) {
@@ -219,11 +221,28 @@ void modstatdb_shutdown(void) {
   free(updatefnbuf);
 }
 
+struct pipef *status_pipes= NULL;
+
 void modstatdb_note(struct pkginfo *pkg) {
   assert(cstatus >= msdbrw_write);
 
   onerr_abort++;
 
+  if (status_pipes) {
+    static struct varbuf *status= NULL;
+    struct pipef *pipef= status_pipes;
+    int r;
+    if (status == NULL) {
+      status = nfmalloc(sizeof(struct varbuf));
+      varbufinit(status);
+    } else
+      varbufreset(status);
+    r= varbufprintf(status, "status: %s: %s\n", pkg->name, statusinfos[pkg->status].name);
+    while (pipef) {
+      write(pipef->fd, status->buf, r);
+      pipef= pipef->next;
+    }
+  }
   varbufreset(&uvb);
   varbufrecord(&uvb, pkg, &pkg->installed);
   if (fwrite(uvb.buf, 1, uvb.used, importanttmp) != uvb.used)
@@ -243,7 +262,10 @@ void modstatdb_note(struct pkginfo *pkg) {
 
   nextupdate++;  
 
-  if (nextupdate > MAXUPDATES) checkpoint();
+  if (nextupdate > MAXUPDATES) {
+    checkpoint();
+    nextupdate= 0;
+  }
 
   createimptmp();
 
