@@ -31,25 +31,36 @@
 static int convert_string
 (const char *filename, int lno, const char *what, int otherwise,
  FILE *warnto, int *warncount, const struct pkginfo *pigp,
- const char *startp, const struct namevalue *nvip,
+ const char *startp, const struct namevalue *ivip,
  const char **endpp) 
 {
   const char *ep;
-  int c, l;
+  int c, l = 0;
+  struct namevalue *nvip= NULL;
+  memcpy(&nvip,&ivip,sizeof(struct namevalue *));
 
   ep= startp;
-  if (!*ep) parseerr(0,filename,lno, warnto,warncount,pigp,0, _("%s is missing"),what);
-  while ((c= *ep) && !isspace(c)) ep++;
-  l= (int)(ep-startp);
-  while (nvip->name && (strncasecmp(nvip->name,startp,l) || nvip->name[l])) nvip++;
+  if (!*ep) parseerr(NULL,filename,lno, warnto,warncount,pigp,0, _("%s is missing"),what);
+  while (nvip->name) {
+    if ((l= nvip->length) == 0) {
+      l= strlen(nvip->name);
+      nvip->length= (const int)l;
+    }
+    if (strncasecmp(nvip->name,startp,l) || nvip->name[l])
+      nvip++;
+    else
+      break;
+  }
   if (!nvip->name) {
     if (otherwise != -1) return otherwise;
-    parseerr(0,filename,lno, warnto,warncount,pigp,0, _("`%.*s' is not allowed for %s"),
+    parseerr(NULL,filename,lno, warnto,warncount,pigp,0, _("`%.*s' is not allowed for %s"),
              l > 50 ? 50 : l, startp, what);
   }
+  ep = startp + l;
+  c = *ep;
   while (isspace(c)) c= *++ep;
   if (c && !endpp)
-    parseerr(0,filename,lno, warnto,warncount,pigp,0, _("junk after %s"),what);
+    parseerr(NULL,filename,lno, warnto,warncount,pigp,0, _("junk after %s"),what);
   if (endpp) *endpp= ep;
   return nvip->value;
 }
@@ -58,8 +69,8 @@ void f_name(struct pkginfo *pigp, struct pkginfoperfile *pifp, enum parsedbflags
             const char *filename, int lno, FILE *warnto, int *warncount,
             const char *value, const struct fieldinfo *fip) {
   const char *e;
-  if ((e= illegal_packagename(value,0)) != 0)
-    parseerr(0,filename,lno, warnto,warncount,pigp,0, _("invalid package name (%.250s)"),e);
+  if ((e= illegal_packagename(value,NULL)) != NULL)
+    parseerr(NULL,filename,lno, warnto,warncount,pigp,0, _("invalid package name (%.250s)"),e);
   pigp->name= nfstrsave(value);
   findpackage(value); /* We discard the value from findpackage, but calling it
                        * forces an entry in the hash table to be made if it isn't
@@ -77,10 +88,10 @@ void f_filecharf(struct pkginfo *pigp, struct pkginfoperfile *pifp,
   int allowextend;
 
   if (!*value)
-    parseerr(0,filename,lno, warnto,warncount,pigp,0,
+    parseerr(NULL,filename,lno, warnto,warncount,pigp,0,
              _("empty file details field `%s'"),fip->name);
   if (!(flags & pdb_recordavailable))
-    parseerr(0,filename,lno, warnto,warncount,pigp,0,
+    parseerr(NULL,filename,lno, warnto,warncount,pigp,0,
              _("file details field `%s' not allowed in status file"),fip->name);
   allowextend= !pigp->files;
   fdpp= &pigp->files;
@@ -91,20 +102,20 @@ void f_filecharf(struct pkginfo *pigp, struct pkginfoperfile *pifp,
     fdp= *fdpp;
     if (!fdp) {
       if (!allowextend)
-        parseerr(0,filename,lno, warnto,warncount,pigp,0, _("too many values "
+        parseerr(NULL,filename,lno, warnto,warncount,pigp,0, _("too many values "
                  "in file details field `%s' (compared to others)"),fip->name);
       fdp= nfmalloc(sizeof(struct filedetails));
-      fdp->next= 0;
-      fdp->name= fdp->msdosname= fdp->size= fdp->md5sum= 0;
+      fdp->next= NULL;
+      fdp->name= fdp->msdosname= fdp->size= fdp->md5sum= NULL;
       *fdpp= fdp;
     }
-    FILEFFIELD(fdp,fip->integer,char*)= cpos;
+    FILEFFIELD(fdp,fip->integer,const char*)= cpos;
     fdpp= &fdp->next;
     while (*space && isspace(*space)) space++;
     cpos= space;
   }
   if (*fdpp)
-    parseerr(0,filename,lno, warnto,warncount,pigp,0, _("too few values "
+    parseerr(NULL,filename,lno, warnto,warncount,pigp,0, _("too few values "
              "in file details field `%s' (compared to others)"),fip->name);
 }
 
@@ -122,7 +133,7 @@ void f_boolean(struct pkginfo *pigp, struct pkginfoperfile *pifp,
   pifp->essential=
     *value ? convert_string(filename,lno,_("yes/no in `essential' field"), -1,
                             warnto,warncount,pigp,
-                            value,booleaninfos,0)
+                            value,booleaninfos,NULL)
            : 0;
 }
 
@@ -141,7 +152,7 @@ void f_priority(struct pkginfo *pigp, struct pkginfoperfile *pifp,
   if (!*value) return;
   pigp->priority= convert_string(filename,lno,"word in `priority' field", pri_other,
                              warnto,warncount,pigp,
-                             value,priorityinfos,0);
+                             value,priorityinfos,NULL);
   if (pigp->priority == pri_other) pigp->otherpriority= nfstrsave(value);
 }
 
@@ -152,7 +163,7 @@ void f_status(struct pkginfo *pigp, struct pkginfoperfile *pifp,
   const char *ep;
 
   if (flags & pdb_rejectstatus)
-    parseerr(0,filename,lno, warnto,warncount,pigp,0,
+    parseerr(NULL,filename,lno, warnto,warncount,pigp,0,
              _("value for `status' field not allowed in this context"));
   if (flags & pdb_recordavailable) return;
   
@@ -165,7 +176,7 @@ void f_status(struct pkginfo *pigp, struct pkginfoperfile *pifp,
     pigp->eflag &= ~eflagf_obsoletehold;
   }
   pigp->status= convert_string(filename,lno,_("third (status) word in `status' field"), -1,
-                               warnto,warncount,pigp, ep,statusinfos,0);
+                               warnto,warncount,pigp, ep,statusinfos,NULL);
 }
 
 void f_version(struct pkginfo *pigp, struct pkginfoperfile *pifp,
@@ -175,7 +186,7 @@ void f_version(struct pkginfo *pigp, struct pkginfoperfile *pifp,
   const char *emsg;
   
   emsg= parseversion(&pifp->version,value);
-  if (emsg) parseerr(0,filename,lno, warnto,warncount,pigp,0, _("error "
+  if (emsg) parseerr(NULL,filename,lno, warnto,warncount,pigp,0, _("error "
                      "in Version string `%.250s': %.250s"),value,emsg);
 }  
 
@@ -185,7 +196,7 @@ void f_revision(struct pkginfo *pigp, struct pkginfoperfile *pifp,
                 const char *value, const struct fieldinfo *fip) {
   char *newversion;
 
-  parseerr(0,filename,lno, warnto,warncount,pigp,1,
+  parseerr(NULL,filename,lno, warnto,warncount,pigp,1,
            _("obsolete `Revision' or `Package-Revision' field used"));
   if (!*value) return;
   if (pifp->version.revision && *pifp->version.revision) {
@@ -203,12 +214,12 @@ void f_configversion(struct pkginfo *pigp, struct pkginfoperfile *pifp,
   const char *emsg;
   
   if (flags & pdb_rejectstatus)
-    parseerr(0,filename,lno, warnto,warncount,pigp,0,
+    parseerr(NULL,filename,lno, warnto,warncount,pigp,0,
              _("value for `config-version' field not allowed in this context"));
   if (flags & pdb_recordavailable) return;
 
   emsg= parseversion(&pigp->configversion,value);
-  if (emsg) parseerr(0,filename,lno, warnto,warncount,pigp,0, _("error "
+  if (emsg) parseerr(NULL,filename,lno, warnto,warncount,pigp,0, _("error "
                      "in Config-Version string `%.250s': %.250s"),value,emsg);
 }
 
@@ -219,31 +230,34 @@ void f_conffiles(struct pkginfo *pigp, struct pkginfoperfile *pifp,
   struct conffile **lastp, *newlink;
   const char *endent, *endfn;
   int c, namelen, hashlen;
+  char *newptr;
   
   lastp= &pifp->conffiles;
   while (*value) {
     c= *value++;
     if (c == '\n') continue;
-    if (c != ' ') parseerr(0,filename,lno, warnto,warncount,pigp,0, _("value for"
+    if (c != ' ') parseerr(NULL,filename,lno, warnto,warncount,pigp,0, _("value for"
                            " `conffiles' has line starting with non-space `%c'"), c);
     for (endent= value; (c= *endent)!=0 && c != '\n'; endent++);
     for (endfn= endent; *endfn != ' '; endfn--);
     if (endfn <= value+1 || endfn >= endent-1)
-      parseerr(0,filename,lno, warnto,warncount,pigp,0,
+      parseerr(NULL,filename,lno, warnto,warncount,pigp,0,
                _("value for `conffiles' has malformatted line `%.*s'"),
                (int)(endent-value > 250 ? 250 : endent-value), value);
     newlink= nfmalloc(sizeof(struct conffile));
     value= skip_slash_dotslash(value);
     namelen= (int)(endfn-value);
-    if (namelen <= 0) parseerr(0,filename,lno, warnto,warncount,pigp,0,
+    if (namelen <= 0) parseerr(NULL,filename,lno, warnto,warncount,pigp,0,
                                _("root or null directory is listed as a conffile"));
-    newlink->name= nfmalloc(namelen+2);
-    newlink->name[0]= '/';
-    memcpy(newlink->name+1,value,namelen);
-    newlink->name[namelen+1]= 0;
-    hashlen= (int)(endent-endfn)-1; newlink->hash= nfmalloc(hashlen+1);
-    memcpy(newlink->hash,endfn+1,hashlen); newlink->hash[hashlen]= 0;
-    newlink->next =0;
+    newptr = nfmalloc(namelen+2);
+    newptr[0]= '/';
+    memcpy(newptr+1,value,namelen);
+    newptr[namelen+1]= 0;
+    newlink->name= newptr;
+    hashlen= (int)(endent-endfn)-1; newptr= nfmalloc(hashlen+1);
+    memcpy(newptr,endfn+1,hashlen); newptr[hashlen]= 0;
+    newlink->hash= newptr;
+    newlink->next =NULL;
     *lastp= newlink;
     lastp= &newlink->next;
     value= endent;
@@ -256,50 +270,60 @@ void f_dependency(struct pkginfo *pigp, struct pkginfoperfile *pifp,
                   const char *value, const struct fieldinfo *fip) {
   char c1, c2;
   const char *p, *emsg;
-  struct varbuf depname, version;
+  const char *depnamestart, *versionstart;
+  int depnamelength, versionlength;
+  static int depnameused= 0, versionused= 0;
+  static char *depname= NULL, *version= NULL;
+
   struct dependency *dyp, **ldypp;
   struct deppossi *dop, **ldopp;
 
   if (!*value) return; /* empty fields are ignored */
-  varbufinit(&depname); varbufinit(&version);
   p= value;
   ldypp= &pifp->depends; while (*ldypp) ldypp= &(*ldypp)->next;
   for (;;) { /* loop creating new struct dependency's */
     dyp= nfmalloc(sizeof(struct dependency));
-    dyp->up= 0; /* Set this to zero for now, as we don't know what our real
+    dyp->up= NULL; /* Set this to zero for now, as we don't know what our real
                  * struct pkginfo address (in the database) is going to be yet.
                  */
-    dyp->next= 0; *ldypp= dyp; ldypp= &dyp->next;
-    dyp->list= 0; ldopp= &dyp->list;
+    dyp->next= NULL; *ldypp= dyp; ldypp= &dyp->next;
+    dyp->list= NULL; ldopp= &dyp->list;
     dyp->type= fip->integer;
     for (;;) { /* loop creating new struct deppossi's */
-      varbufreset(&depname);
+      depnamestart= p;
+/* skip over package name characters */
       while (*p && !isspace(*p) && *p != '(' && *p != ',' && *p != '|') {
-        varbufaddc(&depname,*p); p++;
+	p++;
       }
-      varbufaddc(&depname,0);
-      if (!*depname.buf)
-        parseerr(0,filename,lno, warnto,warncount,pigp,0, _("`%s' field, missing"
+      depnamelength= p - depnamestart ;
+      if (depnamelength >= depnameused) {
+	depnameused= depnamelength;
+	depname= realloc(depname,depnamelength+1);
+      }
+      strncpy(depname, depnamestart, depnamelength);
+      *(depname + depnamelength)= 0;
+      if (!*depname)
+        parseerr(NULL,filename,lno, warnto,warncount,pigp,0, _("`%s' field, missing"
                  " package name, or garbage where package name expected"), fip->name);
-      emsg= illegal_packagename(depname.buf,0);
-      if (emsg) parseerr(0,filename,lno, warnto,warncount,pigp,0, _("`%s' field,"
+      emsg= illegal_packagename(depname,NULL);
+      if (emsg) parseerr(NULL,filename,lno, warnto,warncount,pigp,0, _("`%s' field,"
                          " invalid package name `%.255s': %s"),
-                         fip->name,depname.buf,emsg);
+                         fip->name,depname,emsg);
       dop= nfmalloc(sizeof(struct deppossi));
       dop->up= dyp;
-      dop->ed= findpackage(depname.buf);
-      dop->next= 0; *ldopp= dop; ldopp= &dop->next;
-      dop->nextrev= 0; /* Don't link this (which is after all only `newpig' from
-      dop->backrev= 0;  * the main parsing loop in parsedb) into the depended on
+      dop->ed= findpackage(depname);
+      dop->next= NULL; *ldopp= dop; ldopp= &dop->next;
+      dop->nextrev= NULL; /* Don't link this (which is after all only `newpig' from */
+      dop->backrev= NULL; /* the main parsing loop in parsedb) into the depended on
                         * packages' lists yet.  This will be done later when we
                         * install this (in parse.c).  For the moment we do the
                         * `forward' links in deppossi (`ed') only, and the backward
                         * links from the depended on packages to dop are left undone.
                         */
       dop->cyclebreak= 0;
+/* skip whitespace after packagename */
       while (isspace(*p)) p++;
-      if (*p == '(') {
-        varbufreset(&version);
+      if (*p == '(') {			/* if we have a versioned relation */
         p++; while (isspace(*p)) p++;
         c1= *p;
         if (c1 == '<' || c1 == '>') {
@@ -312,50 +336,67 @@ void f_dependency(struct pkginfo *pigp, struct pkginfoperfile *pifp,
             dop->verrel |= (dvrf_strict | dvrf_builtup);
             p++;
           } else if (c2 == '<' || c2 == '>') {
-            parseerr(0,filename,lno, warnto,warncount,pigp,0,
+            parseerr(NULL,filename,lno, warnto,warncount,pigp,0,
                     _("`%s' field, reference to `%.255s':\n"
                     " bad version relationship %c%c"),
-                    fip->name,depname.buf,c1,c2);
+                    fip->name,depname,c1,c2);
             dop->verrel= dvr_none;
           } else {
-            parseerr(0,filename,lno, warnto,warncount,pigp,1,
+            parseerr(NULL,filename,lno, warnto,warncount,pigp,1,
                      _("`%s' field, reference to `%.255s':\n"
                      " `%c' is obsolete, use `%c=' or `%c%c' instead"),
-                     fip->name,depname.buf,c1,c1,c1,c1);
+                     fip->name,depname,c1,c1,c1,c1);
             dop->verrel |= (dvrf_orequal | dvrf_builtup);
           }
         } else if (c1 == '=') {
           dop->verrel= dvr_exact;
           p++;
         } else {
-          parseerr(0,filename,lno, warnto,warncount,pigp,1,
+          parseerr(NULL,filename,lno, warnto,warncount,pigp,1,
                    _("`%s' field, reference to `%.255s':\n"
                    " implicit exact match on version number, suggest using `=' instead"),
-                   fip->name,depname.buf);
+                   fip->name,depname);
           dop->verrel= dvr_exact;
         }
+	if ((dop->verrel!=dvr_exact) && (fip->integer==dep_provides))
+	  parseerr(NULL,filename,lno,warnto,warncount,pigp,1,
+		  _("Only exact versions may be used for Provides"));
+
         if (!isspace(*p) && !isalnum(*p)) {
-          parseerr(0,filename,lno, warnto,warncount,pigp,1,
+          parseerr(NULL,filename,lno, warnto,warncount,pigp,1,
                    _("`%s' field, reference to `%.255s':\n"
                    " version value starts with non-alphanumeric, suggest adding a space"),
-                   fip->name,depname.buf);
+                   fip->name,depname);
         }
+/* skip spaces between the relation and the version */
         while (isspace(*p)) p++;
+
+	versionstart= p;
         while (*p && *p != ')' && *p != '(') {
-          if (!isspace(*p)) varbufaddc(&version,*p);
+          if (isspace(*p)) break;
           p++;
         }
-        if (*p == '(') parseerr(0,filename,lno, warnto,warncount,pigp,0,
+        while (isspace(*p)) p++;
+        if (*p == '(') parseerr(NULL,filename,lno, warnto,warncount,pigp,0,
                                 _("`%s' field, reference to `%.255s': "
-                                "version contains `('"),fip->name,depname.buf);
-        else if (*p == 0) parseerr(0,filename,lno, warnto,warncount,pigp,0,
+                                "version contains `('"),fip->name,depname);
+	else if (*p != ')') parseerr(NULL,filename,lno, warnto,warncount,pigp,0,
+                                _("`%s' field, reference to `%.255s': "
+                                "version contains ` '"),fip->name,depname);
+        else if (*p == 0) parseerr(NULL,filename,lno, warnto,warncount,pigp,0,
                                    _("`%s' field, reference to `%.255s': "
-                                   "version unterminated"),fip->name,depname.buf);
-        varbufaddc(&version,0);
-        emsg= parseversion(&dop->version,version.buf);
-        if (emsg) parseerr(0,filename,lno, warnto,warncount,pigp,0,
+                                   "version unterminated"),fip->name,depname);
+	versionlength= p - versionstart;
+	if (versionlength >=  versionused) {
+	  versionused= versionlength;
+	  version= realloc(version,versionlength+1);
+	}
+	strncpy(version,  versionstart, versionlength);
+	*(version + versionlength)= 0;
+        emsg= parseversion(&dop->version,version);
+        if (emsg) parseerr(NULL,filename,lno, warnto,warncount,pigp,0,
                            _("`%s' field, reference to `%.255s': "
-                           "error in version: %.255s"),fip->name,depname.buf,emsg);
+                           "error in version: %.255s"),fip->name,depname,emsg);
         p++; while (isspace(*p)) p++;
       } else {
         dop->verrel= dvr_none;
@@ -363,13 +404,13 @@ void f_dependency(struct pkginfo *pigp, struct pkginfoperfile *pifp,
       }
       if (!*p || *p == ',') break;
       if (*p != '|')
-        parseerr(0,filename,lno, warnto,warncount,pigp,0, _("`%s' field, syntax"
+        parseerr(NULL,filename,lno, warnto,warncount,pigp,0, _("`%s' field, syntax"
                  " error after reference to package `%.255s'"),
                  fip->name, dop->ed->name);
       if (fip->integer == dep_conflicts ||
           fip->integer == dep_provides ||
           fip->integer == dep_replaces)
-        parseerr(0,filename,lno, warnto,warncount,pigp,0,
+        parseerr(NULL,filename,lno, warnto,warncount,pigp,0,
                  _("alternatives (`|') not allowed in %s field"),
                  fip->name);
       p++; while (isspace(*p)) p++;
