@@ -3,6 +3,7 @@
  * myopt.c - my very own option parsing
  *
  * Copyright (C) 1994,1995 Ian Jackson <iwj10@cus.cam.ac.uk>
+ * Copyright (C) 2000 Wichert Akkerman <wakkerma@debian.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as
@@ -20,11 +21,69 @@
  */
 
 #include <string.h>
+#include <errno.h>
+#include <ctype.h>
 
 #include <config.h>
 #include <myopt.h>
 #include <dpkg.h>
 
+void myfileopt(const char* fn, const struct cmdinfo* cmdinfos) {
+  FILE* file;
+  char linebuf[MAXDIVERTFILENAME];
+
+  file= fopen(fn, "r");
+  if (!file) {
+    if (errno==ENOENT)
+      return;
+    ohshite(_("failed to open configuration file `%.255s' for reading"), fn);
+  }
+
+  while (fgets(linebuf, sizeof(linebuf), file)) {
+    char* opt;
+    const struct cmdinfo *cip;
+    int l;
+
+    if ((linebuf[0]=='#') || (linebuf[0]=='\n') || (linebuf[0]==0)) continue;
+    l=strlen(linebuf);
+    if (linebuf[l-1]=='\n') linebuf[l-1]=0;
+    for (opt=linebuf;isalnum(*opt)||*opt=='-';opt++) ;
+    if (*opt==0)
+      opt=NULL;
+    else {
+      *opt++=0;
+      if (*opt=='=') opt++;
+      while (isspace(*opt)) opt++;
+    }
+
+    for (cip=cmdinfos; cip->olong || cip->oshort; cip++) {
+      int l;
+      if (!cip->olong) continue;
+      if (!strcmp(cip->olong,linebuf)) break;
+      l=strlen(cip->olong);
+      if ((cip->takesvalue==2) && (linebuf[l]=='-') &&
+	  !opt && !strncmp(linebuf,cip->olong,l)) {
+	opt=linebuf+l+1;
+	break;
+      }
+    }
+
+    if (!cip->olong) ohshite(_("configuration error: unknown option %s"), linebuf);
+
+    if (cip->takesvalue) {
+      if (!opt) ohshite(_("configuration error: %s needs a value"), linebuf);
+      if (cip->call) cip->call(cip,opt);
+      else *cip->sassignto= opt;
+    } else {
+      if (opt) ohshite(_("configuration error: %s does not take a value"), linebuf);
+      if (cip->call) cip->call(cip,NULL);
+      else *cip->iassignto= cip->arg;
+    }
+  }
+  if (ferror(file)) ohshite(_("read error in configuration file `%.255s'"), fn);
+  if (fclose(file)) ohshite(_("error closing configuration file `%.255s'"), fn);
+}
+
 void myopt(const char *const **argvp, const struct cmdinfo *cmdinfos) {
   const struct cmdinfo *cip;
   const char *p, *value;
@@ -35,7 +94,7 @@ void myopt(const char *const **argvp, const struct cmdinfo *cmdinfos) {
     ++(*argvp);
     if (!strcmp(p,"--")) break;
     if (*++p == '-') {
-      ++p; value=0;
+      ++p; value=NULL;
       for (cip= cmdinfos;
            cip->olong || cip->oshort;
            cip++) {
@@ -55,7 +114,7 @@ void myopt(const char *const **argvp, const struct cmdinfo *cmdinfos) {
         else *cip->sassignto= value;
       } else {
         if (value) badusage(_("--%s option does not take a value"),cip->olong);
-        if (cip->call) cip->call(cip,0);
+        if (cip->call) cip->call(cip,NULL);
         else *cip->iassignto= cip->arg;
       }
     } else {
@@ -75,7 +134,7 @@ void myopt(const char *const **argvp, const struct cmdinfo *cmdinfos) {
           else *cip->sassignto= value;
         } else {
           if (*p == '=') badusage(_("-%c option does not take a value"),cip->oshort);
-          if (cip->call) cip->call(cip,0);
+          if (cip->call) cip->call(cip,NULL);
           else *cip->iassignto= cip->arg;
         }
       }
