@@ -56,54 +56,53 @@ void parseerr
 }
 
 const struct namevalue booleaninfos[]= {  /* Note !  These must be in order ! */
-  { "no",                             0                  },
-  { "yes",                            1                  },
-  {  0                                                   }
+  { "no",                             0,                 2 },
+  { "yes",                            1,                 3 },
+  {  NULL                                                  }
 };
 
 const struct namevalue priorityinfos[]= {  /* Note !  These must be in order ! */
-  { "required",                       pri_required     },
-  { "important",                      pri_important    },
-  { "standard",                       pri_standard     },
-  { "recommended",                    pri_recommended  }, /* fixme: obsolete */
-  { "optional",                       pri_optional     },
-  { "extra",                          pri_extra        },
-  { "contrib",                        pri_contrib      }, /* fixme: keep? */
-  { "this is a bug - please report",  pri_other        },
-  { "unknown",                        pri_unknown      },
-  
-  { "base",                           pri_required     }, /* fixme: alias, remove */
-  {  0                                                 }
+  { "required",                       pri_required,     8 },
+  { "important",                      pri_important,    9 },
+  { "standard",                       pri_standard,     8 },
+  { "recommended",                    pri_recommended,  11 }, /* fixme: obsolete */
+  { "optional",                       pri_optional,     8 },
+  { "extra",                          pri_extra,        5 },
+  { "contrib",                        pri_contrib,      7 }, /* fixme: keep? */
+  { "this is a bug - please report",  pri_other,        28 },
+  { "unknown",                        pri_unknown,      7 },
+  { "base",                           pri_required,     4 }, /* fixme: alias, remove */
+  {  NULL                                                 }
 };
 
 const struct namevalue statusinfos[]= {  /* Note !  These must be in order ! */
-  { "not-installed",   stat_notinstalled    },
-  { "unpacked",        stat_unpacked        },
-  { "half-configured", stat_halfconfigured  },
-  { "installed",       stat_installed       },
-  { "half-installed",  stat_halfinstalled   },
-  { "config-files",    stat_configfiles     },
+  { "not-installed",   stat_notinstalled,    13 },
+  { "unpacked",        stat_unpacked,        8 },
+  { "half-configured", stat_halfconfigured,  15, },
+  { "installed",       stat_installed,       9 },
+  { "half-installed",  stat_halfinstalled,   14 },
+  { "config-files",    stat_configfiles,     12 },
   /* These are additional entries for reading only, in any order ... */
-  { "postinst-failed", stat_halfconfigured  }, /* fixme: backwards compat., remove */
-  { "removal-failed",  stat_halfinstalled   }, /* fixme: backwards compat., remove */
-  {  0                                      }
+  { "postinst-failed", stat_halfconfigured,  15 }, /* fixme: backwards compat., remove */
+  { "removal-failed",  stat_halfinstalled,   14 }, /* fixme: backwards compat., remove */
+  {  NULL                                       }
 };
 
 const struct namevalue eflaginfos[]= {  /* Note !  These must be in order ! */
-  { "ok",                      eflagv_ok                 },
-  { "reinstreq",               eflagv_reinstreq          },
-  { "hold",                    eflagv_obsoletehold       },
-  { "hold-reinstreq",          eflagv_obsoleteboth       },
-  {  0                                                   }
+  { "ok",                      eflagv_ok,                2 },
+  { "reinstreq",               eflagv_reinstreq,         9 },
+  { "hold",                    eflagv_obsoletehold,      4 },
+  { "hold-reinstreq",          eflagv_obsoleteboth,      14 },
+  {  NULL                                                   }
 };
 
 const struct namevalue wantinfos[]= {  /* Note !  These must be in order ! */
-  { "unknown",   want_unknown    },
-  { "install",   want_install    },
-  { "hold",      want_hold       },
-  { "deinstall", want_deinstall  },
-  { "purge",     want_purge      },
-  {  0                           }
+  { "unknown",   want_unknown,    7 },
+  { "install",   want_install,    7 },
+  { "hold",      want_hold,       4 },
+  { "deinstall", want_deinstall,  9 },
+  { "purge",     want_purge,      5 },
+  {  NULL                           }
 };
 
 const char *illegal_packagename(const char *p, const char **ep) {
@@ -116,10 +115,10 @@ const char *illegal_packagename(const char *p, const char **ep) {
   if (!*++p) return _("must be at least two characters");
   while ((c= *p++)!=0)
     if (!isalnum(c) && !strchr(alsoallowed,c)) break;
-  if (!c) return 0;
+  if (!c) return NULL;
   if (isspace(c) && ep) {
     while (isspace(*p)) p++;
-    *ep= p; return 0;
+    *ep= p; return NULL;
   }
   snprintf(buf, sizeof(buf),
           _("character `%c' not allowed - only letters, digits and %s allowed"),
@@ -134,7 +133,7 @@ const struct nickname nicknames[]= {
   { "Class",             "Priority"    },
   { "Package-Revision",  "Revision"    },
   { "Package_Revision",  "Revision"    },
-  {  0                                 }
+  {  NULL                              }
 };
 
 int informativeversion(const struct versionrevision *version) {
@@ -189,10 +188,22 @@ const char *versiondescribe
 
 const char *parseversion(struct versionrevision *rversion, const char *string) {
   char *hyphen, *colon, *eepochcolon;
+  const char *end, *ptr;
   unsigned long epoch;
 
   if (!*string) return _("version string is empty");
-  
+
+  /* trim leading and trailing space */
+  while (*string && (*string == ' ' || *string == '\t') ) string++;
+  /* string now points to the first non-whitespace char */
+  end = string;
+  /* find either the end of the string, or a whitespace char */
+  while (*end && *end != ' ' && *end != '\t' ) end++;
+  /* check for extra chars after trailing space */
+  ptr = end;
+  while (*ptr && ( *ptr == ' ' || *ptr == '\t' ) ) ptr++;
+  if (*ptr) return _("version string has embedded spaces");
+
   colon= strchr(string,':');
   if (colon) {
     epoch= strtoul(string,&eepochcolon,10);
@@ -203,23 +214,26 @@ const char *parseversion(struct versionrevision *rversion, const char *string) {
   } else {
     rversion->epoch= 0;
   }
-  rversion->version= nfstrsave(string);
+  rversion->version= nfstrnsave(string,end-string+1);
   hyphen= strrchr(rversion->version,'-');
   if (hyphen) *hyphen++= 0;
-  rversion->revision= hyphen ? hyphen : nfstrsave("");
+  rversion->revision= hyphen ? hyphen : "";
   
-  return 0;
+  return NULL;
 }
 
 void parsemustfield
 (FILE *file, const char *filename, int lno,
  FILE *warnto, int *warncount,
  const struct pkginfo *pigp, int warnonly,
- char **value, const char *what) 
+ const char **value, const char *what) 
 {
+  static char *empty = NULL;
+  if (!empty)
+    empty= nfstrsave("");
   if (!*value) {
     parseerr(file,filename,lno, warnto,warncount,pigp,warnonly, _("missing %s"),what);
-    *value= nfstrsave("");
+    *value= empty;
   } else if (!**value) {
     parseerr(file,filename,lno, warnto,warncount,pigp,warnonly,
              _("empty value for %s"),what);
