@@ -27,13 +27,24 @@
 #include <dpkg.h>
 #include <dpkg-db.h>
 
-void varbufaddc(struct varbuf *v, int c) {
+inline void varbufaddc(struct varbuf *v, int c) {
   if (v->used >= v->size) varbufextend(v);
   v->buf[v->used++]= c;
 }
 
-void varbufprintf(struct varbuf *v, const char *fmt, ...) {
-  int ou, r;
+void varbufdupc(struct varbuf *v, int c, ssize_t n) {
+  char *b = v->buf + v->used;
+  v->used += n;
+  if (v->used >= v->size) varbufextend(v);
+ 
+  while(n) {
+    *b= c;
+    b++; n--;
+  }
+}
+
+int varbufprintf(struct varbuf *v, const char *fmt, ...) {
+  unsigned int ou, r;
   va_list al;
 
   ou= v->used;
@@ -47,11 +58,28 @@ void varbufprintf(struct varbuf *v, const char *fmt, ...) {
     if (r < 0) r= (v->size-ou+1) * 2;
     v->used= ou+r;
   } while (r >= v->size-ou-1);
+  return r;
 }
 
-void varbufaddstr(struct varbuf *v, const char *s) {
-  int l, ou;
-  l= strlen(s);
+int varbufvprintf(struct varbuf *v, const char *fmt, va_list va) {
+  unsigned int ou, r;
+  va_list al;
+
+  ou= v->used;
+  v->used+= strlen(fmt);
+
+  do {
+    varbufextend(v);
+    __va_copy(al, va);
+    r= vsnprintf(v->buf+ou,v->size-ou,fmt,al);
+    if (r < 0) r= (v->size-ou+1) * 2;
+    v->used= ou+r;
+  } while (r >= v->size-ou-1);
+  return r;
+}
+
+void varbufaddbuf(struct varbuf *v, const void *s, const int l) {
+  int ou;
   ou= v->used;
   v->used += l;
   if (v->used >= v->size) varbufextend(v);
@@ -61,7 +89,7 @@ void varbufaddstr(struct varbuf *v, const char *s) {
 void varbufinit(struct varbuf *v) {
   /* NB: dbmodify.c does its own init to get a big buffer */
   v->size= v->used= 0;
-  v->buf= 0;
+  v->buf= NULL;
 }
 
 void varbufreset(struct varbuf *v) {
@@ -80,5 +108,5 @@ void varbufextend(struct varbuf *v) {
 }
 
 void varbuffree(struct varbuf *v) {
-  free(v->buf); v->buf=0; v->size=0; v->used=0;
+  free(v->buf); v->buf=NULL; v->size=0; v->used=0;
 }
