@@ -3,6 +3,7 @@
  * archives.c - actions that process archive files, mainly unpack
  *
  * Copyright (C) 1994,1995 Ian Jackson <iwj10@cus.cam.ac.uk>
+ * Copyright (C) 2000 Wichert Akkerman <wakkerma@debian.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as
@@ -32,6 +33,10 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
+#include <obstack.h>
+#define obstack_chunk_alloc m_malloc
+#define obstack_chunk_free free
+
 #include <config.h>
 #include <dpkg.h>
 #include <dpkg-db.h>
@@ -46,6 +51,102 @@
 struct pkginfo *conflictor[20];
 int cflict_index = 0;
 
+/* snprintf(3) doesn't work if format contains %.<nnn>s and an argument has
+ * invalid char for locale, then it returns -1.
+ * ohshite() is ok, but fd_fd_copy(), which is used in tarobject() in this
+ * file, is not ok, because
+ * - fd_fd_copy() == buffer_copy_setup() [include/dpkg.h]
+ * - buffer_copy_setup() uses varbufvprintf(&v, desc, al); [lib/mlib.c]
+ * - varbufvpprintf() fails and memory exausted, because it call
+ *    fmt = "backend dpkg-deb during `%.255s'
+ *    arg may contain some invalid char, for example,
+ *    /usr/share/doc/console-tools/examples/unicode/\342\231\252\342\231\254
+ *   in console-tools.
+ *   In this case, if user uses some locale which doesn't support \342\231...,
+ *   vsnprintf() always return -1 and varbufextend() again and again
+ *   and memory exausted and die.
+ *
+ * So, we need to escape invalid char, probably as in
+ * tar-1.13.19/lib/quotearg.c: quotearg_buffer_restyled()
+ * but here I escape all 8bit chars, in order to be simple.
+ * - ukai@debian.or.jp
+ */
+static char *
+quote_filename(char *buf, int size, char *s)
+{
+       char *r = buf;
+       while (size > 0) {
+               switch (*s) {
+               case '\0':
+                       *buf = '\0';
+                       return r;
+               case '\\':
+                       *buf++ = '\\';
+                       *buf++ = '\\';
+                       size -= 2;
+                       break;
+               default:
+                       if (((*s)&0x80) == 0) {
+                               *buf++ = *s++;
+                               --size;
+                       } else {
+                               if (size > 4) {
+                                       sprintf(buf, "\\%03o",
+                                               *(unsigned char *)s);
+                                       size -= 4;
+                                       buf += 4;
+                                       s++;
+                               } else {
+                                       /* buffer full */
+                                       *buf = '\0'; /* XXX */
+                                       return s;
+                               }
+                       }
+               }
+       }
+       *buf = '\0'; /* XXX */
+       return r;
+
+}
+
+/* special routine to handle partial reads from the tarfile */
+static int safe_read(int fd, void *buf, int len)
+{
+  int r, have= 0;
+  char *p = (char *)buf;
+  while (have < len) {
+    if ((r= read(fd,p,len-have))==-1) {
+      if (errno==EINTR || errno==EAGAIN) continue;
+      return r;
+    }
+    if (r==0)
+      break;
+    have+= r;
+    p+= r;
+  }
+  return have;
+}
+
+static struct obstack tar_obs;
+static int tarobs_init= 0;
+
+/* ensure the obstack is properly initialized */
+static void ensureobstackinit() {
+
+  if (!tarobs_init) {
+    obstack_init(&tar_obs);
+    tarobs_init= 1;
+  }
+}
+
+/* destroy the obstack */
+static void destroyobstack() {
+  if (tarobs_init) {
+    obstack_free(&tar_obs, 0);
+    tarobs_init= 0;
+  }
+}
+
 int filesavespackage(struct fileinlist *file, struct pkginfo *pkgtobesaved,
                      struct pkginfo *pkgbeinginstalled) {
   struct pkginfo *divpkg, *thirdpkg;
@@ -114,16 +215,10 @@ void cu_pathname(int argc, void **argv) {
   ensure_pathname_nonexisting((char*)(argv[0]));
 } 
 
-void cu_backendpipe(int argc, void **argv) {
-  FILE *f= *(FILE**)argv[0];
-  if (f) fclose(f);
-} 
-
 int tarfileread(void *ud, char *buf, int len) {
   struct tarcontext *tc= (struct tarcontext*)ud;
   int r;
-  r= fread(buf,1,len,tc->backendpipe);
-  if (r != len && ferror(tc->backendpipe))
+  if ((r= safe_read(tc->backendpipe,buf,len)) == -1)
     ohshite(_("error reading from dpkg-deb pipe"));
   return r;
 }
@@ -164,10 +259,12 @@ static void newtarobject_utime(const char *path, struct TarInfo *ti) {
     ohshite(_("error setting timestamps of `%.255s'"),ti->Name);
 }
 
-static void newtarobject_allmodes(const char *path, struct TarInfo *ti) {
-  if (chown(path,ti->UserID,ti->GroupID))
+static void newtarobject_allmodes(const char *path, struct TarInfo *ti, struct filestatoverride* statoverride) {
+  if (chown(path,
+	    statoverride ? statoverride->uid : ti->UserID,
+	    statoverride ? statoverride->gid : ti->GroupID))
     ohshite(_("error setting ownership of `%.255s'"),ti->Name);
-  if (chmod(path,ti->Mode & ~S_IFMT))
+  if (chmod(path,(statoverride ? statoverride->mode : ti->Mode) & ~S_IFMT))
     ohshite(_("error setting permissions of `%.255s'"),ti->Name);
   newtarobject_utime(path,ti);
 }
@@ -211,33 +308,35 @@ int unlinkorrmdir(const char *filename) {
         filename, r ? strerror(e) : "OK");
   errno= e; return r;
 }
-     
+
 int tarobject(struct TarInfo *ti) {
   static struct varbuf conffderefn, hardlinkfn, symlinkfn;
   const char *usename;
     
   struct tarcontext *tc= (struct tarcontext*)ti->UserData;
-  int statr, fd, r, i, existingdirectory;
+  int statr, fd, i, existingdirectory;
+  size_t r;
   struct stat stab, stabd;
-  size_t sz, wsz;
-  FILE *thefile;
   char databuf[TARBLKSZ];
   struct fileinlist *nifd;
   struct pkginfo *divpkg, *otherpkg;
   struct filepackages *packageslump;
+  mode_t am;
+
+  ensureobstackinit();
 
   /* Append to list of files.
    * The trailing / put on the end of names in tarfiles has already
    * been stripped by TarExtractor (lib/tarfn.c).
    */
-  nifd= m_malloc(sizeof(struct fileinlist));
+  nifd= obstack_alloc(&tar_obs, sizeof(struct fileinlist));
   nifd->namenode= findnamenode(ti->Name, 0);
   nifd->next= 0; *tc->newfilesp= nifd; tc->newfilesp= &nifd->next;
   nifd->namenode->flags |= fnnf_new_inarchive;
 
   debug(dbg_eachfile,
-        _("tarobject ti->Name=`%s' Mode=%lo owner=%u.%u Type=%d(%c)"
-        " ti->LinkName=`%s' namenode=`%s' flags=%o instead=`%s'"),
+        "tarobject ti->Name=`%s' Mode=%lo owner=%u.%u Type=%d(%c)"
+        " ti->LinkName=`%s' namenode=`%s' flags=%o instead=`%s'",
         ti->Name, (long)ti->Mode, (unsigned)ti->UserID, (unsigned)ti->GroupID, ti->Type,
         ti->Type == '\0' ? '_' :
         ti->Type >= '0' && ti->Type <= '6' ? "-hlcbdp"[ti->Type - '0'] : '?',
@@ -355,7 +454,7 @@ int tarobject(struct TarInfo *ti) {
           printf(_("Replacing files in old package %s ...\n"),otherpkg->name);
           otherpkg->clientdata->replacingfilesandsaid= 1;
         } else {
-          if (S_ISDIR(stab.st_mode)) {
+          if (!statr && S_ISDIR(stab.st_mode)) {
             forcibleerr(fc_overwritedir, _("trying to overwrite directory `%.250s' "
                         "in package %.250s with nondirectory"),
                         nifd->namenode->name,otherpkg->name);
@@ -390,55 +489,48 @@ int tarobject(struct TarInfo *ti) {
   /* Extract whatever it is as .dpkg-new ... */
   switch (ti->Type) {
   case NormalFile0: case NormalFile1:
-    fd= open(fnamenewvb.buf, O_CREAT|O_EXCL|O_WRONLY,
-             ti->Mode & (S_IRUSR|S_IRGRP|S_IROTH));
-    if (fd < 0) ohshite("unable to create `%.255s'",ti->Name);
-    thefile= fdopen(fd,"w");
-    if (!thefile) { close(fd); ohshite(_("unable to fdopen for `%.255s'"),ti->Name); }
-    push_cleanup(cu_closefile,ehflag_bombout, 0,0, 1,(void*)thefile);
+    /* We create the file with mode 0 to make sure nobody can do anything with
+     * it until we apply the proper mode, which might be a statoverride.
+     */
+    fd= open(fnamenewvb.buf, (O_CREAT|O_EXCL|O_WRONLY), 0);
+    if (fd < 0) ohshite(_("unable to create `%.255s'"),ti->Name);
+    push_cleanup(cu_closefd,ehflag_bombout, 0,0, 1,&fd);
     debug(dbg_eachfiledetail,"tarobject NormalFile[01] open size=%lu",
           (unsigned long)ti->Size);
-    for (sz= ti->Size; sz > 0; sz -= wsz) {
-      wsz= sz > TARBLKSZ ? TARBLKSZ : sz;
-      r= fread(databuf,1,TARBLKSZ,tc->backendpipe);
-      if (r<TARBLKSZ) {
-        if (ferror(tc->backendpipe)) {
-          ohshite(_("error reading dpkg-deb during `%.255s'"),ti->Name);
-        } else {
-          errno= 0;
-          return -1;
-        }
-      }
-      if (fwrite(databuf,1,wsz,thefile) != wsz)
-        ohshite(_("error writing to `%.255s'"),ti->Name);
+    { char fnamebuf[256];
+    fd_fd_copy(tc->backendpipe, fd, ti->Size, _("backend dpkg-deb during `%.255s'"),quote_filename(fnamebuf,256,ti->Name));
     }
-    if (fchown(fd,ti->UserID,ti->GroupID))
+    r= ti->Size % TARBLKSZ;
+    if (r > 0) r= safe_read(tc->backendpipe,databuf,TARBLKSZ - r);
+    if (nifd->namenode->statoverride) 
+      debug(dbg_eachfile, "tarobject ... stat override, uid=%d, gid=%d, mode=%04o",
+			  nifd->namenode->statoverride->uid,
+			  nifd->namenode->statoverride->gid,
+			  nifd->namenode->statoverride->mode);
+    if (fchown(fd,
+	    nifd->namenode->statoverride ? nifd->namenode->statoverride->uid : ti->UserID,
+	    nifd->namenode->statoverride ? nifd->namenode->statoverride->gid : ti->GroupID))
       ohshite(_("error setting ownership of `%.255s'"),ti->Name);
-    /* We flush the stream here to avoid any future
-     * writes, which will mask any setuid or setgid
-     * attemps by fchmod below.
-     */
-    if (fflush(thefile) == EOF)
-      ohshite(_("error flushing `%.255s'"),ti->Name);
-    if (fchmod(fd,ti->Mode & ~S_IFMT))
+    am=(nifd->namenode->statoverride ? nifd->namenode->statoverride->mode : ti->Mode) & ~S_IFMT;
+    if (fchmod(fd,am))
       ohshite(_("error setting permissions of `%.255s'"),ti->Name);
-    pop_cleanup(ehflag_normaltidy); /* thefile= fdopen(fd) */
-    if (fclose(thefile))
+    pop_cleanup(ehflag_normaltidy); /* fd= open(fnamenewvb.buf) */
+    if (close(fd))
       ohshite(_("error closing/writing `%.255s'"),ti->Name);
     newtarobject_utime(fnamenewvb.buf,ti);
     break;
   case FIFO:
-    if (mkfifo(fnamenewvb.buf,ti->Mode & S_IFMT))
+    if (mkfifo(fnamenewvb.buf,0))
       ohshite(_("error creating pipe `%.255s'"),ti->Name);
     debug(dbg_eachfiledetail,"tarobject FIFO");
-    newtarobject_allmodes(fnamenewvb.buf,ti);
+    newtarobject_allmodes(fnamenewvb.buf,ti, nifd->namenode->statoverride);
     break;
   case CharacterDevice: case BlockDevice:
-    if (mknod(fnamenewvb.buf,ti->Mode & S_IFMT,ti->Device))
+    if (mknod(fnamenewvb.buf,0,ti->Device))
       ohshite(_("error creating device `%.255s'"),ti->Name);
     debug(dbg_eachfiledetail,"tarobject CharacterDevice|BlockDevice");
-    newtarobject_allmodes(fnamenewvb.buf,ti);
-    break;
+    newtarobject_allmodes(fnamenewvb.buf,ti, nifd->namenode->statoverride);
+    break; 
   case HardLink:
     varbufreset(&hardlinkfn);
     varbufaddstr(&hardlinkfn,instdir); varbufaddc(&hardlinkfn,'/');
@@ -446,7 +538,7 @@ int tarobject(struct TarInfo *ti) {
     if (link(hardlinkfn.buf,fnamenewvb.buf))
       ohshite(_("error creating hard link `%.255s'"),ti->Name);
     debug(dbg_eachfiledetail,"tarobject HardLink");
-    newtarobject_allmodes(fnamenewvb.buf,ti);
+    newtarobject_allmodes(fnamenewvb.buf,ti, nifd->namenode->statoverride);
     break;
   case SymbolicLink:
     /* We've already cheched for an existing directory. */
@@ -454,19 +546,20 @@ int tarobject(struct TarInfo *ti) {
       ohshite(_("error creating symbolic link `%.255s'"),ti->Name);
     debug(dbg_eachfiledetail,"tarobject SymbolicLink creating");
 #ifdef HAVE_LCHOWN
-    if (lchown(fnamenewvb.buf,ti->UserID,ti->GroupID))
+    if (lchown(fnamenewvb.buf,
 #else
-    if (chown(fnamenewvb.buf,ti->UserID,ti->GroupID))
+    if (chown(fnamenewvb.buf,
 #endif
+	    nifd->namenode->statoverride ? nifd->namenode->statoverride->uid : ti->UserID,
+	    nifd->namenode->statoverride ? nifd->namenode->statoverride->gid : ti->GroupID))
       ohshite(_("error setting ownership of symlink `%.255s'"),ti->Name);
     break;
   case Directory:
     /* We've already checked for an existing directory. */
-    if (mkdir(fnamenewvb.buf,
-              ti->Mode & (S_IRUSR|S_IRGRP|S_IROTH | S_IXUSR|S_IXGRP|S_IXOTH)))
+    if (mkdir(fnamenewvb.buf,0))
       ohshite(_("error creating directory `%.255s'"),ti->Name);
     debug(dbg_eachfiledetail,"tarobject Directory creating");
-    newtarobject_allmodes(fnamenewvb.buf,ti);
+    newtarobject_allmodes(fnamenewvb.buf,ti,nifd->namenode->statoverride);
     break;
   default:
     internerr("bad tar type, but already checked");
@@ -508,7 +601,7 @@ int tarobject(struct TarInfo *ti) {
       do {
         varbufextend(&symlinkfn);
         r= readlink(fnamevb.buf,symlinkfn.buf,symlinkfn.size);
-        if (r<0) ohshite("unable to read link `%.255s'",ti->Name);
+        if (r<0) ohshite(_("unable to read link `%.255s'"),ti->Name);
       } while (r == symlinkfn.size);
       symlinkfn.used= r; varbufaddc(&symlinkfn,0);
       if (symlink(symlinkfn.buf,fnametmpvb.buf))
@@ -560,7 +653,8 @@ static int try_remove_can(struct deppossi *pdep,
       }
     }
     pdep->up->up->clientdata->istobe= itb_deconfigure;
-    newdeconf= m_malloc(sizeof(struct packageinlist));
+    ensureobstackinit();
+    newdeconf= obstack_alloc(&tar_obs, sizeof(struct packageinlist));
     newdeconf->next= deconfigure;
     newdeconf->pkg= pdep->up->up;
     deconfigure= newdeconf;
@@ -598,7 +692,7 @@ void check_conflict(struct dependency *dep, struct pkginfo *pkg,
          (((fixbyrm->want != want_install && fixbyrm->want != want_hold) ||
            does_replace(pkg,&pkg->available,fixbyrm)) &&
           (!fixbyrm->installed.essential || fc_removeessential)))) {
-      assert(fixbyrm->clientdata->istobe == itb_normal);
+      assert(fixbyrm->clientdata->istobe == itb_normal || fixbyrm->clientdata->istobe == itb_deconfigure);
       fixbyrm->clientdata->istobe= itb_remove;
       fprintf(stderr, _("dpkg: considering removing %s in favour of %s ...\n"),
               fixbyrm->name, pkg->name);
@@ -658,7 +752,7 @@ void check_conflict(struct dependency *dep, struct pkginfo *pkg,
 	/* if this gets triggered, it means a package has > 10 conflicts/replaces
 	 * pairs, which is the package's fault
 	 */
-	assert(cflict_index < sizeof(conflictor));
+	assert(cflict_index < (int)sizeof(conflictor));
         /* This conflict is OK - we'll remove the conflictor. */
 	conflictor[cflict_index++]= fixbyrm;
         varbuffree(&conflictwhy); varbuffree(&removalwhy);
@@ -688,19 +782,14 @@ void cu_cidir(int argc, void **argv) {
 }  
 
 void cu_fileslist(int argc, void **argv) {
-  struct fileinlist **headp= (struct fileinlist**)argv[0];
-  struct fileinlist *current, *next;
-  for (current= *headp; current; current= next) {
-    next= current->next;
-    free(current);
-  }
+  destroyobstack();
 }  
 
 void archivefiles(const char *const *argv) {
   const char *volatile thisarg;
   const char *const *volatile argp;
   jmp_buf ejbuf;
-  int pi[2], fc, nfiles, c, i;
+  int pi[2], fc, nfiles, c, i, r;
   FILE *pf;
   static struct varbuf findoutput;
   const char **arglist;
@@ -722,30 +811,31 @@ void archivefiles(const char *const *argv) {
     m_pipe(pi);
     if (!(fc= m_fork())) {
       const char *const *ap;
+      char **narglist;
       int i;
       m_dup2(pi[1],1); close(pi[0]); close(pi[1]);
       for (i=0, ap=argv; *ap; ap++, i++);
-      arglist= m_malloc(sizeof(char*)*(i+15));
-      arglist[0]= FIND;
+      narglist= m_malloc(sizeof(char*)*(i+15));
+      narglist[0]= strdup(FIND);
       for (i=1, ap=argv; *ap; ap++, i++) {
         if (strchr(FIND_EXPRSTARTCHARS,(*ap)[0])) {
           char *a;
           a= m_malloc(strlen(*ap)+10);
           strcpy(a,"./");
           strcat(a,*ap);
-          arglist[i]= a;
+          narglist[i]= a;
         } else {
-          arglist[i]= *ap;
+          narglist[i]= strdup(*ap);
         }
       }
-      arglist[i++]= "-follow"; /*  When editing these, make sure that     */
-      arglist[i++]= "-name";   /*  arglist is mallocd big enough, above.  */
-      arglist[i++]= ARCHIVE_FILENAME_PATTERN;
-      arglist[i++]= "-type";
-      arglist[i++]= "f";
-      arglist[i++]= "-print0";
-      arglist[i++]= 0;
-      execvp(FIND, (char* const*)arglist);
+      narglist[i++]= strdup("-follow"); /*  When editing these, make sure that     */
+      narglist[i++]= strdup("-name");   /*  arglist is mallocd big enough, above.  */
+      narglist[i++]= strdup(ARCHIVE_FILENAME_PATTERN);
+      narglist[i++]= strdup("-type");
+      narglist[i++]= strdup("f");
+      narglist[i++]= strdup("-print0");
+      narglist[i++]= 0;
+      execvp(FIND, narglist);
       ohshite(_("failed to exec find for --recursive"));
     }
     close(pi[1]);
@@ -759,7 +849,9 @@ void archivefiles(const char *const *argv) {
     }
     if (ferror(pf)) ohshite(_("error reading find's pipe"));
     if (fclose(pf)) ohshite(_("error closing find's pipe"));
-    waitsubproc(fc,"find",0);
+    r= waitsubproc(fc,"find",PROCNOERR);
+    if(!(r==0 || r==1))
+      ohshit(_("find for --recursive returned unhandled error %i"),r);
 
     if (!nfiles)
       ohshit(_("searched, but found no packages (files matching *.deb)"));
@@ -786,12 +878,17 @@ void archivefiles(const char *const *argv) {
 
   currenttime= time(0);
 
+  varbufinit(&fnamevb);
+  varbufinit(&fnametmpvb);
+  varbufinit(&fnamenewvb);
+
   varbufaddstr(&fnamevb,instdir); varbufaddc(&fnamevb,'/');
   varbufaddstr(&fnametmpvb,instdir); varbufaddc(&fnametmpvb,'/');
   varbufaddstr(&fnamenewvb,instdir); varbufaddc(&fnamenewvb,'/');
   fnameidlu= fnamevb.used;
 
   ensure_diversions();
+  ensure_statoverrides();
   
   while ((thisarg= *argp++) != 0) {
     if (setjmp(ejbuf)) {
@@ -889,3 +986,5 @@ int wanttoinstall(struct pkginfo *pkg, const struct versionrevision *ver, int sa
   }
 }
 
+/* vi: ts=8 sw=2
+ */
