@@ -44,7 +44,7 @@
 
 int conffoptcells[2][2]= { CONFFOPTCELLS };
 
-static void md5hash(struct pkginfo *pkg, char hashbuf[MD5HASHLEN+1], const char *fn);
+static void md5hash(struct pkginfo *pkg, char **hashbuf, const char *fn);
 
 void deferred_configure(struct pkginfo *pkg) {
   /* The algorithm for deciding what to configure first is as follows:
@@ -71,7 +71,7 @@ void deferred_configure(struct pkginfo *pkg) {
   char *cdr2rest;
   int ok, r, useredited, distedited, c, cc, status, c1;
   struct conffile *conff;
-  char currenthash[MD5HASHLEN+1], newdisthash[MD5HASHLEN+1];
+  char *currenthash= 0, *newdisthash= 0;
   struct stat stab;
   enum conffopt what;
   const char *s;
@@ -153,7 +153,7 @@ void deferred_configure(struct pkginfo *pkg) {
         conff->hash= nfstrsave("-");
         continue;
       }
-      md5hash(pkg,currenthash,cdr.buf);
+      md5hash(pkg,&currenthash,cdr.buf);
 
       varbufreset(&cdr2);
       varbufaddstr(&cdr2,cdr.buf);
@@ -166,7 +166,7 @@ void deferred_configure(struct pkginfo *pkg) {
         if (errno == ENOENT) continue;
         ohshite(_("unable to stat new dist conffile `%.250s'"),cdr2.buf);
       }
-      md5hash(pkg,newdisthash,cdr2.buf);
+      md5hash(pkg,&newdisthash,cdr2.buf);
 
       /* Copy the permissions from the installed version to the new
        * distributed version.
@@ -316,7 +316,7 @@ void deferred_configure(struct pkginfo *pkg) {
 	      char cmdbuf[1024];
 	      p= getenv(PAGERENV);
 	      if (!p || !*p) p= DEFAULTPAGER;
-	      sprintf(cmdbuf, "diff -u %.250s %.250s | %.250s", cdr.buf, cdr2.buf, p);
+	      sprintf(cmdbuf, "diff -Nu %.250s %.250s | %.250s", cdr.buf, cdr2.buf, p);
               s= getenv(SHELLENV);
               if (!s || !*s) s= DEFAULTSHELL;
 	      execlp(s,s,"-c", cmdbuf, NULL);
@@ -423,6 +423,8 @@ void deferred_configure(struct pkginfo *pkg) {
 
       conff->hash= nfstrsave(newdisthash);
       modstatdb_note(pkg);
+      free(newdisthash);
+      free(currenthash);
       
     } /* for (conff= ... */
     varbuffree(&cdr);
@@ -521,7 +523,7 @@ int conffderef(struct pkginfo *pkg, struct varbuf *result, const char *in) {
         if (result->buf[r] == '/') r++;
         result->used= r;
         debug(dbg_conffdetail,"conffderef readlink relative to `%.*s'",
-              result->used, result->buf);
+              (int)result->used, result->buf);
       }
       varbufaddstr(result,linkreadbuf);
       varbufaddc(result,0);
@@ -534,44 +536,20 @@ int conffderef(struct pkginfo *pkg, struct varbuf *result, const char *in) {
   }
 }
     
-static void md5hash(struct pkginfo *pkg, char hashbuf[33], const char *fn) {
-  static int fd, p1[2];
-  FILE *file;
-  pid_t c1;
-  int ok;
+static void md5hash(struct pkginfo *pkg, char **hashbuf, const char *fn) {
+  static int fd;
   
   fd= open(fn,O_RDONLY);
   if (fd >= 0) {
-    push_cleanup(cu_closefd,ehflag_bombout, 0,0, 1,(void*)&fd);
-    m_pipe(p1);
-    push_cleanup(cu_closepipe,ehflag_bombout, 0,0, 1,(void*)&p1[0]);
-    if (!(c1= m_fork())) {
-      m_dup2(fd,0); m_dup2(p1[1],1); close(p1[0]); close(p1[1]);
-      execlp(MD5SUM,MD5SUM,(char*)0);
-      ohshite(_("failed to exec md5sum"));
-    }
-    close(p1[1]); close(fd);
-    file= fdopen(p1[0],"r");
-    push_cleanup(cu_closefile,ehflag_bombout, 0,0, 1,(void*)file);
-    if (!file) ohshite(_("unable to fdopen for md5sum of `%.250s'"),fn);
-    ok= 1;
-    memset(hashbuf,0,33);
-    if (fread(hashbuf,1,32,file) != 32) ok=0;
-    if (getc(file) != '\n') ok=0;
-    if (getc(file) != EOF) ok=0;
-    waitsubproc(c1,"md5sum",0);
-    if (strspn(hashbuf,"0123456789abcdef") != 32) ok=0;
-    if (ferror(file)) ohshite(_("error reading pipe from md5sum"));
-    if (fclose(file)) ohshite(_("error closing pipe from md5sum"));
-    pop_cleanup(ehflag_normaltidy); /* file= fdopen(p1[0]) */
-    pop_cleanup(ehflag_normaltidy); /* m_pipe() */
+    push_cleanup(cu_closefd,ehflag_bombout, 0,0, 1,&fd);
+    fd_md5(fd, hashbuf, -1, _("md5hash"));
     pop_cleanup(ehflag_normaltidy); /* fd= open(cdr.buf) */
-    if (!ok) ohshit(_("md5sum gave malformatted output `%.250s'"),hashbuf);
+    close(fd);
   } else if (errno == ENOENT) {
-    strcpy(hashbuf,NONEXISTENTFLAG);
+    *hashbuf= strdup(NONEXISTENTFLAG);
   } else {
     fprintf(stderr, _("dpkg: %s: warning - unable to open conffile %s for hash: %s\n"),
             pkg->name, fn, strerror(errno));
-    strcpy(hashbuf,"-");
+    *hashbuf= strdup("-");
   }
 }      
