@@ -45,6 +45,7 @@ static int foundcyclebroken(struct cyclesofarlink *thislink,
   const char *postinstfilename;
   struct stat stab;
 
+  if(!possi) return 0;
   /* We're investigating the dependency `possi' to see if it
    * is part of a loop.  To this end we look to see whether the
    * depended-on package is already one of the packages whose
@@ -119,6 +120,7 @@ int findbreakcycle(struct pkginfo *pkg, struct cyclesofarlink *sofar) {
          * still the one we use.
          */
         if (foundcyclebroken(&thislink,sofar,provider,possi)) return 1;
+	if (foundcyclebroken(&thislink,sofar,possi->ed,provider->installed.depended)) return 1;
       }
     }
   }
@@ -133,6 +135,7 @@ void describedepcon(struct varbuf *addto, struct dependency *dep) {
   case dep_predepends:  varbufaddstr(addto, _(" pre-depends on ")); break;
   case dep_recommends:  varbufaddstr(addto, _(" recommends "));     break;
   case dep_conflicts:   varbufaddstr(addto, _(" conflicts with ")); break;
+  case dep_suggests:    varbufaddstr(addto, _(" suggests ")); break;
   case dep_enhances:    varbufaddstr(addto, _(" enhances "));       break;
   default:              internerr("unknown deptype");
   }
