@@ -42,8 +42,8 @@
 #include "main.h"
 
 int pkglistqsortcmp(const void *a, const void *b) {
-  struct pkginfo *pa= *(struct pkginfo**)a;
-  struct pkginfo *pb= *(struct pkginfo**)b;
+  const struct pkginfo *pa= *(const struct pkginfo**)a;
+  const struct pkginfo *pb= *(const struct pkginfo**)b;
   return strcmp(pa->name,pb->name);
 }
 
@@ -156,8 +156,10 @@ void listpackages(const char *const *argv) {
         if (fnmatch(thisarg,pkg->name,0)) continue;
         list1package(pkg,&head); found++;
       }
-      if (!found)
+      if (!found) {
         fprintf(stderr,_("No packages found matching %s.\n"),thisarg);
+	nerrs++;
+      }
     }
   }
   if (ferror(stdout)) werr("stdout");
@@ -176,7 +178,7 @@ static int bsyn_reinstreq(struct pkginfo *pkg, const struct badstatinfo *bsi) {
 
 static int bsyn_status(struct pkginfo *pkg, const struct badstatinfo *bsi) {
   if (pkg->eflag &= eflagf_reinstreq) return 0;
-  return pkg->status == bsi->val;
+  return (int)pkg->status == bsi->val;
 }
 
 static const struct badstatinfo badstatinfos[]= {
@@ -262,6 +264,7 @@ static int yettobeunpacked(struct pkginfo *pkg, const char **thissect) {
   default:
     internerr("unknown status checking for unpackedness");
   }
+  return 0;
 }
 
 void unpackchk(const char *const *argv) {
@@ -410,6 +413,7 @@ void searchfiles(const char *const *argv) {
     }
     if (!found) {
       fprintf(stderr,_("dpkg: %s not found.\n"),thisarg);
+      nerrs++;
       if (ferror(stderr)) werr("stderr");
     } else {
       if (ferror(stdout)) werr("stdout");
@@ -445,7 +449,7 @@ void enqperpackage(const char *const *argv) {
           !pkg->files &&
           pkg->want == want_unknown &&
           !informative(pkg,&pkg->installed)) {
-        printf(_("Package `%s' is not installed and no info is available.\n"),pkg->name);
+        fprintf(stderr,_("Package `%s' is not installed and no info is available.\n"),pkg->name);
         failures++;
       } else {
         writerecord(stdout, "<stdout>", pkg, &pkg->installed);
@@ -454,7 +458,7 @@ void enqperpackage(const char *const *argv) {
 
     case act_printavail:
       if (!informative(pkg,&pkg->available)) {
-        printf(_("Package `%s' is not available.\n"),pkg->name);
+        fprintf(stderr,_("Package `%s' is not available.\n"),pkg->name);
         failures++;
       } else {
         writerecord(stdout, "<stdout>", pkg, &pkg->available);
@@ -464,7 +468,7 @@ void enqperpackage(const char *const *argv) {
     case act_listfiles:
       switch (pkg->status) {
       case stat_notinstalled: 
-        printf(_("Package `%s' is not installed.\n"),pkg->name);
+        fprintf(stderr,_("Package `%s' is not installed.\n"),pkg->name);
         failures++;
         break;
         
@@ -500,8 +504,9 @@ void enqperpackage(const char *const *argv) {
   }
 
   if (failures) {
-    puts(_("Use dpkg --info (= dpkg-deb --info) to examine archive files,\n"
-         "and dpkg --contents (= dpkg-deb --contents) to list their contents."));
+    nerrs++;
+    fputs(_("Use dpkg --info (= dpkg-deb --info) to examine archive files,\n"
+         "and dpkg --contents (= dpkg-deb --contents) to list their contents.\n"),stderr);
     if (ferror(stdout)) werr("stdout");
   }
 }
@@ -665,8 +670,8 @@ void printarch(const char *const *argv) {
     { 0,0,0 }
   }, *archp;
                   
-  const char *ccompiler, *arch;
-  int p1[2], c;
+  const char *ccompiler, *arch= NULL;
+  int p1[2];
   pid_t c1;
   FILE *ccpipe;
   struct varbuf vb;
@@ -697,8 +702,7 @@ void printarch(const char *const *argv) {
     exit(0);
   }
   close(p1[1]);
-  while ((c= getc(ccpipe)) != EOF) varbufaddc(&vb,c);
-  if (ferror(ccpipe)) ohshite(_("error reading from CC pipe"));
+  fd_vbuf_copy(fileno(ccpipe), &vb, -1, _("error reading from CC pipe"));
   waitsubproc(c1,"gcc --print-libgcc-file-name",0);
   if (!vb.used) badlgccfn(ccompiler,"",_("empty output"));
   varbufaddc(&vb,0);
@@ -707,10 +711,10 @@ void printarch(const char *const *argv) {
   p= strstr(vb.buf,"/gcc-lib/");
   if (!p) badlgccfn(ccompiler,vb.buf,_("no gcc-lib component"));
   p+= 9;
-  q= strchr(p,'-'); if (!q) badlgccfn(ccompiler,vb.buf,_("no hyphen after gcc-lib"));
+  q= strchr(p,'/'); if (!q) badlgccfn(ccompiler,vb.buf,_("no slash after gcc-lib"));
   ll= q-p;
   for (archp=archtable;
-       archp->from && !(strlen(archp->from) == ll && !strncmp(archp->from,p,ll));
+       archp->from && strncmp(archp->from,p,ll);
        archp++);
   switch (cipaction->arg) {
   case act_printarch:    arch= archp->to;  break;
@@ -725,6 +729,7 @@ void printarch(const char *const *argv) {
   if (fflush(stdout)) werr("stdout");
 }
 
+void cmpversions(const char *const *argv) NONRETURNING;
 void cmpversions(const char *const *argv) {
   struct relationinfo {
     const char *string;
@@ -761,12 +766,12 @@ void cmpversions(const char *const *argv) {
   int r;
   
   if (!argv[0] || !argv[1] || !argv[2] || argv[3])
-    badusage(_("--cmpversions takes three arguments:"
+    badusage(_("--compare-versions takes three arguments:"
              " <version> <relation> <version>"));
 
   for (rip=relationinfos; rip->string && strcmp(rip->string,argv[1]); rip++);
 
-  if (!rip->string) badusage(_("--cmpversions bad relation"));
+  if (!rip->string) badusage(_("--compare-versions bad relation"));
 
   if (*argv[0] && strcmp(argv[0],"<unknown>")) {
     emsg= parseversion(&a,argv[0]);
