@@ -69,8 +69,11 @@ struct filenamenode *namenodetouse(struct filenamenode *namenode, struct pkginfo
 
 void checkpath(void) {
 /* Verify that some programs can be found in the PATH. */
-  static const char *const checklist[]= {
-    "ldconfig", "start-stop-daemon", "install-info", "update-rc.d", 0
+  static const char *const checklist[]= { "ldconfig", 
+#ifdef USE_START_STOP_DAEMON
+    "start-stop-daemon",
+#endif    
+    "install-info", "update-rc.d", 0
   };
 
   struct stat stab;
@@ -81,7 +84,7 @@ void checkpath(void) {
   long l;
 
   path= getenv("PATH");
-  if (!path) fputs(_("dpkg - warning: PATH is not set.\n"), stderr);
+  if (!path) ohshit(_("dpkg - error: PATH is not set.\n"));
   buf=(char*)m_malloc(strlen(path)+2+strlen("start-stop-daemon"));
   
   for (clp=checklist; *clp; clp++) {
@@ -110,7 +113,7 @@ void checkpath(void) {
 
 void ensure_package_clientdata(struct pkginfo *pkg) {
   if (pkg->clientdata) return;
-  pkg->clientdata= nfmalloc(sizeof(struct pkginfoperfile));
+  pkg->clientdata= nfmalloc(sizeof(struct perpackagestate));
   pkg->clientdata->istobe= itb_normal;
   pkg->clientdata->fileslistvalid= 0;
   pkg->clientdata->files= 0;
@@ -132,8 +135,8 @@ void cu_closedir(int argc, void **argv) {
 }
 
 void cu_closefd(int argc, void **argv) {
-  int *ip= (int*)(argv[0]);
-  close(*ip);
+  int ip= *(int*)argv;
+  close(ip);
 }
 
 int ignore_depends(struct pkginfo *pkg) {
@@ -170,7 +173,7 @@ static const char* preexecscript(const char *path, char *const *argv) {
    * none of the stuff here will work if admindir isn't inside instdir
    * as expected. - fixme
    */
-  int instdirl;
+  size_t instdirl;
 
   if (*instdir) {
     if (chroot(instdir)) ohshite(_("failed to chroot to `%.250s'"),instdir);
@@ -192,7 +195,8 @@ static char *const *vbuildarglist(const char *scriptname, va_list ap) {
   int i;
 
   i=0;
-  bufs[i++]= (char*)scriptname; /* yes, cast away const because exec wants it that way */
+  if(bufs[0]) free(bufs[0]);
+  bufs[i++]= strdup(scriptname); /* yes, cast away const because exec wants it that way */
   for (;;) {
     assert(i < PKGSCRIPTMAXARGS);
     nextarg= va_arg(ap,char*);
@@ -212,7 +216,7 @@ static char *const *buildarglist(const char *scriptname, ...) {
   return arglist;
 }
 
-#define NSCRIPTCATCHSIGNALS sizeof(script_catchsignallist)/sizeof(int)-1
+#define NSCRIPTCATCHSIGNALS (int)(sizeof(script_catchsignallist)/sizeof(int)-1)
 static int script_catchsignallist[]= { SIGQUIT, SIGINT, 0 };
 static struct sigaction script_uncatchsignal[NSCRIPTCATCHSIGNALS];
 
@@ -249,15 +253,36 @@ static void setexecute(const char *path, struct stat *stab) {
   if (!chmod(path,0755)) return;
   ohshite(_("unable to set execute permissions on `%.250s'"),path);
 }
+static int do_script(const char *pkg, const char *scriptname, const char *scriptpath, struct stat *stab, char *const arglist[], const char *desc, const char *name, int warn) {
+  const char *scriptexec;
+  int c1, r;
+  setexecute(scriptpath,stab);
+
+  c1= m_fork();
+  if (!c1) {
+    const char **narglist;
+    for (r=0; arglist[r]; r++) ;
+    narglist=nfmalloc((r+1)*sizeof(char*));
+    for (r=1; arglist[r-1]; r++)
+      narglist[r]= arglist[r];
+    scriptexec= preexecscript(scriptpath,narglist);
+    narglist[0]= scriptexec;
+    execv(scriptexec,narglist);
+    ohshite(desc,name);
+  }
+  script_catchsignals(); /* This does a push_cleanup() */
+  r= waitsubproc(c1,name,warn);
+  pop_cleanup(ehflag_normaltidy);
+  return r;
+}
 
 int maintainer_script_installed(struct pkginfo *pkg, const char *scriptname,
                                 const char *description, ...) {
   /* all ...'s are const char*'s */
-  const char *scriptpath, *scriptexec;
+  const char *scriptpath;
   char *const *arglist;
   struct stat stab;
   va_list ap;
-  int c1;
   char buf[100];
 
   scriptpath= pkgadminfile(pkg,scriptname);
@@ -273,29 +298,18 @@ int maintainer_script_installed(struct pkginfo *pkg, const char *scriptname,
     }
     ohshite(_("unable to stat installed %s script `%.250s'"),description,scriptpath);
   }
-  setexecute(scriptpath,&stab);
-  c1= m_fork();
-  if (!c1) {
-    scriptexec= preexecscript(scriptpath,arglist);
-    execv(scriptexec,arglist);
-    ohshite(_("unable to execute %s"),buf);
-  }
-  script_catchsignals(); /* This does a push_cleanup() */
-  waitsubproc(c1,buf,0);
-  pop_cleanup(ehflag_normaltidy);
-
+  do_script(pkg->name, scriptname, scriptpath, &stab, arglist, _("unable to execute %s"), buf, 0);
   ensure_diversions();
   return 1;
 }
   
-int maintainer_script_new(const char *scriptname, const char *description,
+int maintainer_script_new(const char *pkgname,
+			  const char *scriptname, const char *description,
                           const char *cidir, char *cidirrest, ...) {
   char *const *arglist;
-  const char *scriptexec;
   struct stat stab;
   va_list ap;
   char buf[100];
-  int c1;
   
   va_start(ap,cidirrest);
   arglist= vbuildarglist(scriptname,ap);
@@ -310,17 +324,7 @@ int maintainer_script_new(const char *scriptname, const char *description,
     }
     ohshite(_("unable to stat new %s script `%.250s'"),description,cidir);
   }
-  setexecute(cidir,&stab);
-  c1= m_fork();
-  if (!c1) {
-    scriptexec= preexecscript(cidir,arglist);
-    execv(scriptexec,arglist);
-    ohshite(_("unable to execute new %s"),buf);
-  }
-  script_catchsignals(); /* This does a push_cleanup() */
-  waitsubproc(c1,buf,0);
-  pop_cleanup(ehflag_normaltidy);
-
+  do_script(pkgname, scriptname, cidir, &stab, arglist, _("unable to execute new %s"), buf, 0);
   ensure_diversions();
   return 1;
 }
@@ -329,12 +333,10 @@ int maintainer_script_alternative(struct pkginfo *pkg,
                                   const char *scriptname, const char *description,
                                   const char *cidir, char *cidirrest,
                                   const char *ifok, const char *iffallback) {
-  const char *oldscriptpath, *scriptexec;
+  const char *oldscriptpath;
   char *const *arglist;
   struct stat stab;
-  int c1, n, status;
   char buf[100];
-  pid_t r;
 
   oldscriptpath= pkgadminfile(pkg,scriptname);
   arglist= buildarglist(scriptname,
@@ -352,27 +354,8 @@ int maintainer_script_alternative(struct pkginfo *pkg,
             _("dpkg: warning - unable to stat %s `%.250s': %s\n"),
             buf,oldscriptpath,strerror(errno));
   } else {
-    setexecute(oldscriptpath,&stab);
-    c1= m_fork();
-    if (!c1) {
-      scriptexec= preexecscript(oldscriptpath,arglist);
-      execv(scriptexec, arglist);
-      ohshite(_("unable to execute %s"),buf);
-    }
-    script_catchsignals(); /* This does a push_cleanup() */
-    while ((r= waitpid(c1,&status,0)) == -1 && errno == EINTR);
-    if (r != c1) ohshite(_("wait for %s failed"),buf);
-    pop_cleanup(ehflag_normaltidy);
-    if (WIFEXITED(status)) {
-      n= WEXITSTATUS(status); if (!n) return 1;
-      fprintf(stderr, _("dpkg: warning - %s returned error exit status %d\n"),buf,n);
-    } else if (WIFSIGNALED(status)) {
-      n= WTERMSIG(status);
-      fprintf(stderr, _("dpkg: warning - %s killed by signal (%s)%s\n"),
-              buf, strsignal(n), WCOREDUMP(status) ? ", core dumped" : "");
-    } else {
-      ohshit(_("%s failed with unknown wait status code %d"),buf,status);
-    }
+    if (!do_script(pkg->name, scriptname, oldscriptpath, &stab, arglist, _("unable to execute %s"), buf, PROCWARN))
+      return 1;
     ensure_diversions();
   }
   fprintf(stderr, _("dpkg - trying script from the new package instead ...\n"));
@@ -391,17 +374,7 @@ int maintainer_script_alternative(struct pkginfo *pkg,
       ohshite(_("unable to stat %s `%.250s'"),buf,cidir);
   }
 
-  setexecute(cidir,&stab);
-
-  c1= m_fork();
-  if (!c1) {
-    scriptexec= preexecscript(cidir,arglist);
-    execv(scriptexec, arglist);
-    ohshite(_("unable to execute %s"),buf);
-  }
-  script_catchsignals(); /* This does a push_cleanup() */
-  waitsubproc(c1,buf,0);
-  pop_cleanup(ehflag_normaltidy);
+  do_script(pkg->name, scriptname, cidir, &stab, arglist, _("unable to execute %s"), buf, 0);
   fprintf(stderr, _("dpkg: ... it looks like that went OK.\n"));
 
   ensure_diversions();
@@ -451,7 +424,7 @@ int isdirectoryinuse(struct filenamenode *file, struct pkginfo *pkg) {
   return 0;
 }
 
-void oldconffsetflags(struct conffile *searchconff) {
+void oldconffsetflags(const struct conffile *searchconff) {
   struct filenamenode *namenode;
   
   while (searchconff) {
