@@ -67,6 +67,7 @@ Usage: \n\
   dpkg --update-avail <Packages-file>      replace available packages info\n\
   dpkg --merge-avail <Packages-file>       merge with info from file\n\
   dpkg --clear-avail                       erase existing available info\n\
+  dpkg --command-fd <n>                    pass commands in on this file descriptor\n\
   dpkg --forget-old-unavail                forget uninstalled unavailable pkgs\n\
   dpkg -s|--status <package-name> ...      display package status details\n\
   dpkg -p|--print-avail <package-name> ... display available version details\n\
@@ -96,13 +97,14 @@ Options:\n\
   -E|--skip-same-version     Skip packages whose same version is installed\n\
   -G|--refuse-downgrade      Skip packages with earlier version than installed\n\
   -B|--auto-deconfigure      Install even if it would break some other package\n\
-  --largemem | --smallmem    Optimise for large (>4Mb) or small (<4Mb) RAM use\n\
+  --no-debsig                Do no try to verify package signatures\n\
   --no-act                   Just say what we would do - don't do it\n\
   -D|--debug=<octal>         Enable debugging - see -Dhelp or --debug=help\n\
+  --status-fd <n>            Send status change updates to file descriptor <n>\n\
   --ignore-depends=<package>,... Ignore dependencies involving <package>\n\
   --force-...                    Override problems - see --force-help\n\
   --no-force-...|--refuse-...    Stop when problems encountered\n\
-  dpkg --abort-after <n>     abort after encountering <n> errors\n\
+  --abort-after <n>              Abort after encountering <n> errors\n\
 \n\
 Comparison operators for --compare-versions are:\n\
  lt le eq ne ge gt       (treat no version as earlier than any version);\n\
@@ -127,14 +129,15 @@ Options marked [*] produce a lot of output - pipe it through `less' or `more' !"
 
 const struct cmdinfo *cipaction= 0;
 int f_pending=0, f_recursive=0, f_alsoselect=1, f_skipsame=0, f_noact=0;
-int f_autodeconf=0, f_largemem=0;
+int f_autodeconf=0, f_nodebsig=0;
 unsigned long f_debug=0;
 /* Change fc_overwrite to 1 to enable force-overwrite by default */
-int fc_downgrade=1, fc_configureany=0, fc_hold=0, fc_removereinstreq=0, fc_overwrite=1;
+int fc_downgrade=1, fc_configureany=0, fc_hold=0, fc_removereinstreq=0, fc_overwrite=0;
 int fc_removeessential=0, fc_conflicts=0, fc_depends=0, fc_dependsversion=0;
 int fc_autoselect=1, fc_badpath=0, fc_overwritediverted=0, fc_architecture=0;
 int fc_nonroot=0, fc_overwritedir=0, fc_conff_new=0, fc_conff_miss=0;
 int fc_conff_old=0, fc_conff_def=0;
+int fc_badverify = 0;
 
 int errabort = 50;
 const char *admindir= ADMINDIR;
@@ -164,12 +167,15 @@ static const struct forceinfo {
   { "overwrite-diverted",  &fc_overwritediverted        },
   { "overwrite-dir",       &fc_overwritedir             },
   { "architecture",        &fc_architecture             },
+  { "bad-verify",          &fc_badverify                },
   {  0                                                  }
 };
 
+static void helponly(const struct cmdinfo *cip, const char *value) NONRETURNING;
 static void helponly(const struct cmdinfo *cip, const char *value) {
   usage(); exit(0);
 }
+static void versiononly(const struct cmdinfo *cip, const char *value) NONRETURNING;
 static void versiononly(const struct cmdinfo *cip, const char *value) {
   printversion(); exit(0);
 }
@@ -180,6 +186,10 @@ static void setaction(const struct cmdinfo *cip, const char *value) {
   cipaction= cip;
 }
 
+static void setobsolete(const struct cmdinfo *cip, const char *value) {
+  fprintf(stderr, _("Warning: obsolete option `--%s'\n"),cip->olong);
+}
+
 static void setdebug(const struct cmdinfo *cpi, const char *value) {
   char *endp;
 
@@ -254,9 +264,27 @@ static void setinteger(const struct cmdinfo *cip, const char *value) {
   *cip->iassignto= v;
 }
 
-static void setforce(const struct cmdinfo *cip, const char *value) {
+
+static void setstatuspipe(const struct cmdinfo *cip, const char *value) {
+  static struct pipef *lastpipe= NULL;
+  unsigned long v;
+  char *ep;
+
+  if (lastpipe) {
+    lastpipe->next= nfmalloc(sizeof(struct pipef));
+    lastpipe= lastpipe->next;
+  } else {
+    lastpipe= status_pipes= nfmalloc(sizeof(struct pipef));
+  }
+  v= strtoul(value,&ep,0);
+  if (*ep || v > INT_MAX)
+    badusage(_("invalid integer for --%s: `%.250s'"),cip->olong,value);
+  lastpipe->fd= v;
+}
+
+void setforce(const struct cmdinfo *cip, const char *value) {
   const char *comma;
-  int l;
+  size_t l;
   const struct forceinfo *fip;
 
   if (!strcmp(value,"help")) {
@@ -264,14 +292,16 @@ static void setforce(const struct cmdinfo *cip, const char *value) {
   warn but continue:  --force-<thing>,<thing>,...\n\
   stop with error:    --refuse-<thing>,<thing>,... | --no-force-<thing>,...\n\
  Forcing things:\n\
+  all                    Set all force options\n\
   auto-select [*]        (De)select packages to install (remove) them\n\
   downgrade [*]          Replace a package with a lower version\n\
   configure-any          Configure any package which may help this one\n\
   hold                   Process incidental packages even when on hold\n\
   bad-path               PATH is missing important programs, problems likely\n\
   not-root               Try to (de)install things even when not root\n\
-  overwrite[*]           Overwrite a file from one package with another\n\
+  overwrite              Overwrite a file from one package with another\n\
   overwrite-diverted     Overwrite a diverted file with an undiverted version\n\
+  bad-verify             Install a package even if it fails authenticity check\n\
   depends-version [!]    Turn dependency version problems into warnings\n\
   depends [!]            Turn all dependency problems into warnings\n\
   confnew [!]            Always use the new config files, don't prompt\n\
@@ -299,8 +329,13 @@ Forcing options marked [*] are enabled by default.\n"),
     for (fip=forceinfos; fip->name; fip++)
       if (!strncmp(fip->name,value,l) && strlen(fip->name)==l) break;
     if (!fip->name)
-      badusage(_("unknown force/refuse option `%.*s'"), l<250 ? l : 250, value);
-    *fip->opt= cip->arg;
+      if(!strncmp("all",value,l))
+	for (fip=forceinfos; fip->name; fip++)
+	  *fip->opt= cip->arg;
+      else
+	badusage(_("unknown force/refuse option `%.*s'"), l<250 ? (int)l : 250, value);
+    else
+      *fip->opt= cip->arg;
     if (!comma) break;
     value= ++comma;
   }
@@ -314,6 +349,7 @@ static const char *const passlongopts[]= {
 static const char passshortopts[]= "bceIfxX";
 static const char okpassshortopts[]= "D";
 
+void commandfd(const char *const *argv);
 static const struct cmdinfo cmdinfos[]= {
   /* This table has both the action entries in it and the normal options.
    * The action entries are made with the ACTION macro, as they all
@@ -321,6 +357,9 @@ static const struct cmdinfo cmdinfos[]= {
    */
 #define ACTION(longopt,shortopt,code,function) \
  { longopt, shortopt, 0,0,0, setaction, code, 0, (voidfnp)function }
+#define OBSOLETE(longopt,shortopt) \
+ { longopt, shortopt, 0,0,0, setobsolete, 0, 0, 0 }
+
   ACTION( "install",                        'i', act_install,              archivefiles    ),
   ACTION( "unpack",                          0,  act_unpack,               archivefiles    ),
   ACTION( "record-avail",                   'A', act_avail,                archivefiles    ),
@@ -349,17 +388,22 @@ static const struct cmdinfo cmdinfos[]= {
   ACTION( "print-installation-architecture", 0,  act_printinstarch,        printinstarch   ),
   ACTION( "predep-package",                  0,  act_predeppackage,        predeppackage   ),
   ACTION( "compare-versions",                0,  act_cmpversions,          cmpversions     ),
+/*
+  ACTION( "command-fd",                   'c', act_commandfd,   commandfd     ),
+*/
   
+  { "status-fd",	  0,   1,  0,              0,  setstatuspipe },
   { "pending",           'a',  0,  &f_pending,     0,  0,             1              },
   { "recursive",         'R',  0,  &f_recursive,   0,  0,             1              },
   { "no-act",             0,   0,  &f_noact,       0,  0,             1              },
+  { "no-debsig",          0,   0,  &f_nodebsig,    0,  0,             1              },
   {  0,                  'G',  0,  &fc_downgrade,  0,  0, /* alias for --refuse */ 0 },
   { "selected-only",     'O',  0,  &f_alsoselect,  0,  0,             0              },
   { "no-also-select",    'N',  0,  &f_alsoselect,  0,0,0 /* fixme: remove sometime */ },
   { "skip-same-version", 'E',  0,  &f_skipsame,    0,  0,             1              },
   { "auto-deconfigure",  'B',  0,  &f_autodeconf,  0,  0,             1              },
-  { "largemem",           0,   0,  &f_largemem,    0,  0,             1              },
-  { "smallmem",           0,   0,  &f_largemem,    0,  0,            -1              },
+  OBSOLETE( "largemem", 0 ),
+  OBSOLETE( "smallmem", 0 ),
   { "root",               0,   1,  0, 0,               setroot                       },
   { "abort-after",        0,   1,  &errabort,      0,  setinteger,    0              },
   { "admindir",           0,   1,  0, &admindir,       0                             },
@@ -376,10 +420,110 @@ static const struct cmdinfo cmdinfos[]= {
   {  0,                   0                                                          }
 };
 
+static void execbackend(int argc, const char *const *argv) NONRETURNING;
 static void execbackend(int argc, const char *const *argv) {
-  execvp(BACKEND, (char* const*) argv);
+  char **nargv= malloc(sizeof(char *) * argc + 1);
+  int i;
+  if (!nargv) ohshite(_("couldn't malloc in execbackend"));
+  for (i= 0; i < argc; i++)
+    nargv[i]= strdup(argv[i]);
+  nargv[i]= 0;
+  execvp(BACKEND, nargv);
   ohshite(_("failed to exec dpkg-deb"));
 }
+void commandfd(const char *const *argv) {
+  jmp_buf ejbuf;
+  struct varbuf linevb;
+  const char * pipein;
+  const char **newargs;
+  char *ptr, *endptr;
+  FILE *in;
+  int c, lno, infd, i, skipchar;
+  static void (*actionfunction)(const char *const *argv);
+
+  if ((pipein= *argv++) == NULL) badusage(_("--command-fd takes 1 argument, not 0"));
+  if (*argv) badusage(_("--command-fd only takes 1 argument"));
+  if ((infd= strtol(pipein, (char **)NULL, 10)) == -1)
+    ohshite(_("invalid number for --command-fd"));
+  if ((in= fdopen(infd, "r")) == NULL)
+    ohshite(_("couldn't open `%i' for stream"), infd);
+
+  if (setjmp(ejbuf)) { /* expect warning about possible clobbering of argv */
+    error_unwind(ehflag_bombout); exit(2);
+  }
+  varbufinit(&linevb);
+  for (;;lno= 0) {
+    const char **oldargs= NULL;
+    int argc= 1, mode= 0;
+    lno= 0;
+    push_error_handler(&ejbuf,print_error_fatal,0);
+
+    do { c= getc(in); if (c == '\n') lno++; } while (c != EOF && isspace(c));
+    if (c == EOF) break;
+    if (c == '#') {
+      do { c= getc(in); if (c == '\n') lno++; } while (c != EOF && c != '\n');
+      continue;
+    }
+    varbufreset(&linevb);
+    do {
+      varbufaddc(&linevb,c);
+      c= getc(in);
+      if (c == '\n') lno++;
+      if (isspace(c)) argc++;  /* This isn't fully accurate, but overestimating can't hurt. */
+    } while (c != EOF && c != '\n');
+    if (c == EOF) ohshit(_("unexpected eof before end of line %d"),lno);
+    if (!argc) continue;
+    varbufaddc(&linevb,0);
+printf("line=`%*s'\n",(int)linevb.used,linevb.buf);
+    oldargs= newargs= realloc(oldargs,sizeof(const char *) * (argc + 1));
+    argc= 1;
+    ptr= linevb.buf;
+    endptr= ptr + linevb.used;
+    skipchar= 0;
+    while(ptr < endptr) {
+      if (skipchar) {
+	skipchar= 0;
+      } else if (*ptr == '\\') {
+	memmove(ptr, (ptr+1), (linevb.used-(linevb.buf - ptr)-1));
+	endptr--;
+	skipchar= 1;
+	continue;
+      } else if (isspace(*ptr)) {
+	if (mode == 1) {
+	  *ptr= 0;
+	  mode= 0;
+	}
+      } else {
+	if (mode == 0) {
+	  newargs[argc]= ptr;
+	  argc++;
+	  mode= 1;
+	}
+      }
+      ptr++;
+    }
+    *ptr= 0;
+    newargs[argc++] = 0;
+
+/* We strdup each argument, but never free it, because the error messages
+ * contain references back to these strings.  Freeing them, and reusing
+ * the memory, would make those error messages confusing, to say the
+ * least.
+ */
+    for(i=1;i<argc;i++)
+	if (newargs[i]) newargs[i]=strdup(newargs[i]);
+
+    cipaction= NULL;
+    myopt((const char *const**)&newargs,cmdinfos);
+    if (!cipaction) badusage(_("need an action option"));
+
+    actionfunction= (void (*)(const char* const*))cipaction->farg;
+    actionfunction(newargs);
+    set_error_display(0,0);
+    error_unwind(ehflag_normaltidy);
+  }
+}
+
 
 int main(int argc, const char *const *argv) {
   jmp_buf ejbuf;
@@ -413,6 +557,7 @@ int main(int argc, const char *const *argv) {
     }
   }
 
+  myfileopt(CONFIGDIR "/" DPKG ".cfg", cmdinfos);
   myopt(&argv,cmdinfos);
   if (!cipaction) badusage(_("need an action option"));
 
