@@ -77,7 +77,7 @@ void process_archive(const char *filename) {
   struct dependency *dsearch, *newdeplist, **newdeplistlastp;
   struct dependency *newdep, *dep, *providecheck;
   struct deppossi *psearch, **newpossilastp, *possi, *newpossi, *pdep;
-  FILE *conff, *tmpf;
+  FILE *conff;
   DIR *dsd;
   struct filenamenode *namenode;
   struct dirent *de;
@@ -136,8 +136,33 @@ void process_archive(const char *filename) {
     }
   }
   
+  /* Verify the package. */
+  if (!f_nodebsig && (stat(DEBSIGVERIFY, &stab)==0)) {
+    printf(_("Authenticating %s ...\n"), filename);
+    fflush(stdout);
+    c1 = m_fork();
+    if (!c1) {
+      execl(DEBSIGVERIFY, DEBSIGVERIFY, "-q", filename, NULL);
+      ohshite(_("failed to execl debsig-verify"));
+    } else {
+      int status;
+      waitpid(c1, &status, 0);
+      if (!(WIFEXITED(status) && WEXITSTATUS(status) == 0)) {
+	if (! fc_badverify) {
+	  ohshit(_("Verification on package %s failed!"), filename);
+	} else {
+	  fprintf(stderr, _("Verification on package %s failed,\nbut installing anyway as you request.\n"), filename);
+	}
+      } else {
+	printf(_("passed\n"));
+      }
+    }
+  }
+    
+
   if (f_noact) {
     cidir= cidirtmpnambuf;
+    /* We use tmpnam here, not to get a unique filename, but to get a unique directory. */
     if (!tmpnam(cidir)) ohshite(_("unable to get unique filename for control info"));
     strcat(cidir,"/");
   } else {
@@ -171,8 +196,9 @@ void process_archive(const char *filename) {
     pkg->files= nfmalloc(sizeof(struct filedetails));
     pkg->files->next= 0;
     pkg->files->name= pkg->files->msdosname= pkg->files->md5sum= 0;
-    pkg->files->size= nfmalloc(30);
   }
+  /* Always nfmalloc.  Otherwise, we may overwrite some other field(like md5sum). */
+  pkg->files->size= nfmalloc(30);
   sprintf(pkg->files->size,"%lu",(unsigned long)stab.st_size);
 
   if (cipaction->arg == act_avail) {
@@ -191,12 +217,6 @@ void process_archive(const char *filename) {
   if (!pkg->installed.valid) blankpackageperfile(&pkg->installed);
   assert(pkg->available.valid);
 
-  for (deconpil= deconfigure;
-       deconpil;
-       deconpil= deconpiltemp) {
-    deconpiltemp= deconpil->next;
-    free(deconpil);
-  }
   deconfigure= 0;
   clear_istobes();
 
@@ -270,7 +290,7 @@ void process_archive(const char *filename) {
    * hashes across.
    */
   newconffiles= 0; newconffileslastp= &newconffiles;
-  push_cleanup(cu_fileslist,~0, 0,0, 1,(void*)&newconffiles);
+  push_cleanup(cu_fileslist,~0, 0, 0, 0);
   strcpy(cidirrest,CONFFILESFILE);
   conff= fopen(cidir,"r");
   if (conff) {
@@ -412,19 +432,19 @@ void process_archive(const char *filename) {
   if (oldversionstatus == stat_notinstalled) {
     push_cleanup(cu_preinstverynew,~ehflag_normaltidy, 0,0,
                  3,(void*)pkg,(void*)cidir,(void*)cidirrest);
-    maintainer_script_new(PREINSTFILE, "pre-installation", cidir, cidirrest,
+    maintainer_script_new(pkg->name, PREINSTFILE, "pre-installation", cidir, cidirrest,
                           "install", (char*)0);
   } else if (oldversionstatus == stat_configfiles) {
     push_cleanup(cu_preinstnew,~ehflag_normaltidy, 0,0,
                  3,(void*)pkg,(void*)cidir,(void*)cidirrest);
-    maintainer_script_new(PREINSTFILE, "pre-installation", cidir, cidirrest,
+    maintainer_script_new(pkg->name, PREINSTFILE, "pre-installation", cidir, cidirrest,
                           "install", versiondescribe(&pkg->installed.version,
                                                      vdew_nonambig),
                           (char*)0);
   } else {
     push_cleanup(cu_preinstupgrade,~ehflag_normaltidy, 0,0,
                  4,(void*)pkg,(void*)cidir,(void*)cidirrest,(void*)&oldversionstatus);
-    maintainer_script_new(PREINSTFILE, "pre-installation", cidir, cidirrest,
+    maintainer_script_new(pkg->name, PREINSTFILE, "pre-installation", cidir, cidirrest,
                           "upgrade", versiondescribe(&pkg->installed.version,
                                                      vdew_nonambig),
                           (char*)0);
@@ -512,28 +532,22 @@ void process_archive(const char *filename) {
   close(p1[1]);
 
   newfileslist= 0; tc.newfilesp= &newfileslist;
-  push_cleanup(cu_fileslist,~0, 0,0, 1,(void*)&newfileslist);
+  push_cleanup(cu_fileslist,~0, 0, 0, 0);
   tc.pkg= pkg;
-  tc.backendpipe= fdopen(p1[0],"r");
-  if (!tc.backendpipe) ohshite(_("unable to fdopen dpkg-deb extract pipe"));
-  push_cleanup(cu_backendpipe,~ehflag_bombout, 0,0, 1,(void*)&tc.backendpipe);
+  tc.backendpipe= p1[0];
+  push_cleanup(cu_closefd,~ehflag_bombout, 0,0, 1,&tc.backendpipe);
 
   r= TarExtractor((void*)&tc, &tf);
   if (r) {
     if (errno) {
       ohshite(_("error reading dpkg-deb tar output"));
-    } else if (feof(tc.backendpipe)) {
-      waitsubproc(c1,BACKEND " --fsys-tarfile (EOF)",1);
-      ohshite(_("unexpected EOF in filesystem tarfile - corrupted package archive"));
     } else {
       ohshite(_("corrupted filesystem tarfile - corrupted package archive"));
     }
   }
-  tmpf= tc.backendpipe;
-  tc.backendpipe= 0;
-  while (getc(tmpf) != EOF) ; /* allow for possible trailing zeros */
-  fclose(tmpf);
-  waitsubproc(c1,BACKEND " --fsys-tarfile",1);
+  fd_null_copy(tc.backendpipe,-1,_("dpkg-deb: zap possible trailing zeros"));
+  close(tc.backendpipe);
+  waitsubproc(c1,BACKEND " --fsys-tarfile",PROCPIPE);
 
   if (oldversionstatus == stat_halfinstalled || oldversionstatus == stat_unpacked) {
     /* Packages that were in `installed' and `postinstfailed' have been reduced
@@ -651,7 +665,7 @@ void process_archive(const char *filename) {
     debug(dbg_veryverbose, "process_archive info file `%s'", de->d_name);
     if (de->d_name[0] == '.') continue; /* ignore dotfiles, including `.' and `..' */
     p= strrchr(de->d_name,'.'); if (!p) continue; /* ignore anything odd */
-    if (strlen(pkg->name) != p-de->d_name ||
+    if (strlen(pkg->name) != (size_t)(p-de->d_name) ||
         strncmp(de->d_name,pkg->name,p-de->d_name)) continue;
     debug(dbg_stupidlyverbose, "process_archive info this pkg");
     /* Right do we have one ? */
@@ -889,7 +903,7 @@ void process_archive(const char *filename) {
       debug(dbg_veryverbose, "process_archive info file `%s'", de->d_name);
       if (de->d_name[0] == '.') continue;
       p= strrchr(de->d_name,'.'); if (!p) continue;
-      if (strlen(otherpkg->name) != p-de->d_name ||
+      if (strlen(otherpkg->name) != (size_t)(p-de->d_name) ||
           strncmp(de->d_name,otherpkg->name,p-de->d_name)) continue;
       debug(dbg_stupidlyverbose, "process_archive info this pkg");
       fnvb.used= infodirbaseused;
