@@ -1,16 +1,29 @@
 
+# Global variables:
+# $v                - value parameter to function
+# $sourcepackage    - name of sourcepackage
+# %capit            - map to get properly capitilized version of fields
+# %fi               - map of fields values. keys are of the form "S# key"
+#                     where S is source (L is changelog, C is control)
+#                     and # is an index
+# %p2i              - map from datafile+packagename to index in controlfile
+#                     (used if multiple packages can be listed). Key is
+#                     "S key" where S is the source and key is the packagename
+# %substvar         - map with substitution variables
+
 $parsechangelog= 'dpkg-parsechangelog';
 
 grep($capit{lc $_}=$_, qw(Pre-Depends Standards-Version Installed-Size
-			  Build-Depends Build-Depends-Indep
+			  Build-Depends Build-Depends-Indep 
 			  Build-Conflicts Build-Conflicts-Indep));
 
 
-$substvar{'Format'}= 1.6;
+$substvar{'Format'}= 1.7;
 $substvar{'Newline'}= "\n";
 $substvar{'Space'}= " ";
 $substvar{'Tab'}= "\t";
 $maxsubsts=50;
+$warnable_error= 1;
 
 $progname= $0; $progname= $& if $progname =~ m,[^/]+$,;
 
@@ -39,22 +52,13 @@ if(!defined($getlogin)) {
 }
 
 if (defined ($ENV{'LOGNAME'})) {
-    if (!defined ($getlogin)) {
-        # Hmmm, do we really need this?
-	# warn (sprintf ('no utmp entry available, using value of LOGNAME ("%s")', $ENV{'LOGNAME'})); 
-    } else {
-	if ($getlogin ne $ENV{'LOGNAME'}) { 
-	    warn (sprintf ('utmp entry ("%s") does not match value of LOGNAME ("%s"); using "%s"',
-			   $getlogin, $ENV{'LOGNAME'}, $ENV{'LOGNAME'}));
-	}
-    }
     @fowner = getpwnam ($ENV{'LOGNAME'});
     if (! @fowner) { die (sprintf ('unable to get login information for username "%s"', $ENV{'LOGNAME'})); }
 } elsif (defined ($getlogin)) {
     @fowner = getpwnam ($getlogin);
     if (! @fowner) { die (sprintf ('unable to get login information for username "%s"', $getlogin)); }
 } else {
-    warn (sprintf ('no utmp entry available and LOGNAME not defined; using uid of process (%d)', $<));
+    &warn (sprintf ('no utmp entry available and LOGNAME not defined; using uid of process (%d)', $<));
     @fowner = getpwuid ($<);
     if (! @fowner) { die (sprintf ('unable to get login information for uid %d', $<)); }
 }
@@ -66,13 +70,9 @@ sub capit {
 }
 
 sub findarch {
-   if (exists $ENV{DEB_HOST_ARCH}) {
-      $arch=$ENV{DEB_HOST_ARCH};
-   } else {
-      $arch=`dpkg --print-architecture`;
-      $? && &subprocerr("dpkg --print-architecture");
-       chomp $arch;
-   }
+    $arch=`dpkg-architecture -qDEB_HOST_ARCH`;
+    $? && &subprocerr("dpkg-architecture -qDEB_HOST_ARCH");
+    chomp $arch;
     $substvar{'Arch'}= $arch;
 }
 
@@ -122,6 +122,11 @@ sub outputclose {
         $v =~ m/\n\S/ && &internerr("field $f has newline then non whitespace >$v<");
         $v =~ m/\n[ \t]*\n/ && &internerr("field $f has blank lines >$v<");
         $v =~ m/\n$/ && &internerr("field $f has trailing newline >$v<");
+	if ($dosubstvars) {
+	   $v =~ s/,[\s,]*,/,/g;
+	   $v =~ s/^\s*,\s*//;
+	   $v =~ s/\s*,\s*$//;
+	}
         $v =~ s/\$\{\}/\$/g;
         print("$f: $v\n") || &syserr("write error on control data");
     }
@@ -185,6 +190,7 @@ sub parsecdata {
             length($cf) || &syntax("continued value line not in field");
             $fi{"$source$index $cf"}.= "\n$_";
         } elsif (m/^-----BEGIN PGP/ && $many<0) {
+            $many == -2 && syntax("expected blank line before PGP signature");
             while (<CDATA>) { last if m/^$/; }
             $many= -2;
         } elsif (m/^$/) {
@@ -225,6 +231,7 @@ sub error { die "$progname: error: $_[0]\n"; }
 sub internerr { die "$progname: internal error: $_[0]\n"; }
 sub warn { warn "$progname: warning: $_[0]\n"; }
 sub usageerr { print(STDERR "$progname: @_\n\n"); &usageversion; exit(2); }
+sub warnerror { if ($warnable_error) { &warn( @_ ); } else { &error( @_ ); } }
 
 sub subprocerr {
     local ($p) = @_;
