@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 
-$dpkglibdir= ".";
+$dpkglibdir= "."; # This line modified by Makefile
 $version= '1.3.0'; # This line modified by Makefile
 
 $controlfile= 'debian/control';
@@ -11,6 +11,17 @@ $uploadfilesdir= '..';
 $sourcestyle= 'i';
 $quiet= 0;
 
+# Other global variables used:
+# %f2p             - file to package map
+# %p2f             - package to file map
+#                    has entries for both "packagename" and "packagename architecture"
+# %p2ver           - package to version map
+# %f2sec           - file to section map
+# %f2pri           - file to priority map
+# %sourcedefault   - default values as taken from source (used for Section,
+#                    Priority and Maintainer)
+# $changedby       - person who created this package (as listed in changelog)
+
 use POSIX;
 use POSIX qw(:errno_h :signal_h);
 
@@ -19,20 +30,24 @@ require 'controllib.pl';
 
 sub usageversion {
     print STDERR
-"Debian GNU/Linux dpkg-genchanges $version.  Copyright (C) 1996
-Ian Jackson.  This is free software; see the GNU General Public Licence
-version 2 or later for copying conditions.  There is NO warranty.
+"Debian dpkg-genchanges $version. 
+Copyright (C) 1996 Ian Jackson.
+Copyright (C) 2000,2001 Wichert Akkerman.
+This is free software; see the GNU General Public Licence version 2 or later
+for copying conditions.  There is NO warranty.
 
 Usage: dpkg-genchanges [options ...]
 
 Options:  -b                     binary-only build - no source files
           -B                     arch-specific - no source or arch-indep files
+          -S                     source-only upload
           -c<controlfile>        get control info from this file
           -l<changelogfile>      get per-version info from this file
           -f<fileslistfile>      get .deb files list from this file
           -v<sinceversion>       include all changes later than version
           -C<changesdescription> use change description from this file
-          -m<maintainer>         override changelog's maintainer value
+          -m<maintainer>         override control's maintainer value
+          -e<maintainer>         override changelog's maintainer value
           -u<uploadfilesdir>     directory with files (default is \`..')
           -si (default)          src includes orig for debian-revision 0 or 1
           -sa                    source includes orig src
@@ -49,17 +64,22 @@ Options:  -b                     binary-only build - no source files
 
 $i=100;grep($fieldimps{$_}=$i--,
           qw(Format Date Source Binary Architecture Version
-             Distribution Urgency Maintainer Description Closes Changes
-	     Files));
+             Distribution Urgency Maintainer Changed-By Description 
+	     Closes Changes Files));
 
 while (@ARGV) {
     $_=shift(@ARGV);
     if (m/^-b$/) {
+    	$sourceonly && &usageerr("cannot combine -b or -B and -S");
         $binaryonly= 1;
     } elsif (m/^-B$/) {
+    	$sourceonly && &usageerr("cannot combine -b or -B and -S");
 	$archspecific=1;
 	$binaryonly= 1;
 	print STDERR "$progname: arch-specific upload - not including arch-independent packages\n";
+    } elsif (m/^-S$/) {
+    	$binaryonly && &usageerr("cannot combine -b or -B and -S");
+	$sourceonly= 1;
     } elsif (m/^-s([iad])$/) {
         $sourcestyle= $1;
     } elsif (m/^-q$/) {
@@ -78,6 +98,8 @@ while (@ARGV) {
         $varlistfile= $';
     } elsif (m/^-m/) {
         $forcemaint= $';
+    } elsif (m/^-e/) {
+        $forcechangedby= $';
     } elsif (m/^-F([0-9a-z]+)$/) {
         $changelogformat=$1;
     } elsif (m/^-D([^\=:]+)[=:]/) {
@@ -99,62 +121,68 @@ while (@ARGV) {
 &parsechangelog;
 &parsecontrolfile;
 
-$fileslistfile="./$fileslistfile" if $fileslistfile =~ m/^\s/;
-open(FL,"< $fileslistfile") || &syserr("cannot read files list file");
-while(<FL>) {
-    if (m/^(([-+.0-9a-z]+)_([^_]+)_([-\w]+)\.deb) (\S+) (\S+)$/) {
-        defined($p2f{"$2 $4"}) &&
-            &warn("duplicate files list entry for package $2 (line $.)");
-	$f2p{$1}= $2;
-        $p2f{"$2 $4"}= $1;
-        $p2f{$2}= $1;
-        $p2ver{$2}= $3;
-        defined($f2sec{$1}) &&
-            &warn("duplicate files list entry for file $1 (line $.)");
-        $f2sec{$1}= $5;
-        $f2pri{$1}= $6;
-        push(@fileslistfiles,$1);
-    } elsif (m/^([-+.0-9a-z]+_[^_]+_([-\w]+)\.[a-z0-9.]+) (\S+) (\S+)$/) {
-	# A non-deb package
-	$f2sec{$1}= $3;
-	$f2pri{$1}= $4;
-	push(@archvalues,$2) unless !$2 || $archadded{$2}++;
-	push(@fileslistfiles,$1);
-    } elsif (m/^([-+.,_0-9a-zA-Z]+) (\S+) (\S+)$/) {
-	defined($f2sec{$1}) &&
-            &warn("duplicate files list entry for file $1 (line $.)");
-        $f2sec{$1}= $2;
-        $f2pri{$1}= $3;
-        push(@fileslistfiles,$1);
-    } else {
-        &error("badly formed line in files list file, line $.");
+if (not $sourceonly) {
+    $fileslistfile="./$fileslistfile" if $fileslistfile =~ m/^\s/;
+    open(FL,"< $fileslistfile") || &syserr("cannot read files list file");
+    while(<FL>) {
+	if (m/^(([-+.0-9a-z]+)_([^_]+)_([-\w]+)\.u?deb) (\S+) (\S+)$/) {
+	    defined($p2f{"$2 $4"}) &&
+		&warn("duplicate files list entry for package $2 (line $.)");
+	    $f2p{$1}= $2;
+	    $p2f{"$2 $4"}= $1;
+	    $p2f{$2}= $1;
+	    $p2ver{$2}= $3;
+	    defined($f2sec{$1}) &&
+		&warn("duplicate files list entry for file $1 (line $.)");
+	    $f2sec{$1}= $5;
+	    $f2pri{$1}= $6;
+	    push(@fileslistfiles,$1);
+	} elsif (m/^([-+.0-9a-z]+_[^_]+_([-\w]+)\.[a-z0-9.]+) (\S+) (\S+)$/) {
+	    # A non-deb package
+	    $f2sec{$1}= $3;
+	    $f2pri{$1}= $4;
+	    push(@archvalues,$2) unless !$2 || $archadded{$2}++;
+	    push(@fileslistfiles,$1);
+	} elsif (m/^([-+.,_0-9a-zA-Z]+) (\S+) (\S+)$/) {
+	    defined($f2sec{$1}) &&
+		&warn("duplicate files list entry for file $1 (line $.)");
+	    $f2sec{$1}= $2;
+	    $f2pri{$1}= $3;
+	    push(@fileslistfiles,$1);
+	} else {
+	    &error("badly formed line in files list file, line $.");
+	}
     }
+    close(FL);
 }
-close(FL);
 
 for $_ (keys %fi) {
     $v= $fi{$_};
     if (s/^C //) {
-#print STDERR "G key >$_< value >$v<\n";
 	if (m/^Source$/) { &setsourcepackage; }
-	elsif (m/^Section$|^Priority$/) { $sourcedefault{$_}= $v; }
+	elsif (m/^Section$|^Priority$/i) { $sourcedefault{$_}= $v; }
+	elsif (m/^Maintainer$/i) { $f{$_}= $v; }
 	elsif (s/^X[BS]*C[BS]*-//i) { $f{$_}= $v; }
-	elsif (m/|^X[BS]+-|^Standards-Version$|^Maintainer$/i) { }
+	elsif (m/|^X[BS]+-|^Standards-Version$/i) { }
 	else { &unknown('general section of control info file'); }
     } elsif (s/^C(\d+) //) {
-#print STDERR "P key >$_< value >$v<\n";
 	$i=$1; $p=$fi{"C$i Package"}; $a=$fi{"C$i Architecture"};
-	if (!defined($p2f{$p})) {
+	if (!defined($p2f{$p}) && not $sourceonly) {
 	    if ($a eq 'any' || ($a eq 'all' && !$archspecific) ||
 		grep($_ eq $substvar{'Arch'}, split(/\s+/, $a))) {
-		&error("package $p in control file but not in files list");
+		&warn("package $p in control file but not in files list");
+		next;
 	    }
 	} else {
 	    $p2arch{$p}=$a;
 	    $f=$p2f{$p};
 	    if (m/^Description$/) {
 		$v=$` if $v =~ m/\n/;
-		push(@descriptions,sprintf("%-10s - %-.65s",$p,$v));
+		if ($f =~ m/\.udeb$/) {
+			push(@descriptions,sprintf("%-10s - %-.65s (udeb)",$p,$v));
+		} else {
+			push(@descriptions,sprintf("%-10s - %-.65s",$p,$v));
+		}
 	    } elsif (m/^Section$/) {
 		$f2seccf{$f}= $v;
 	    } elsif (m/^Priority$/) {
@@ -162,10 +190,14 @@ for $_ (keys %fi) {
 	    } elsif (s/^X[BS]*C[BS]*-//i) {
 		$f{$_}= $v;
 	    } elsif (m/^Architecture$/) {
-		if ($v eq 'any' || grep($_ eq $arch, split(/\s+/, $v))) {
-		    $v= $arch;
-		} elsif ($v ne 'all') {
-		    $v= '';
+		if (not $sourceonly) {
+		    if ($v eq 'any' || grep($_ eq $arch, split(/\s+/, $v))) {
+			$v= $arch;
+		    } elsif ($v ne 'all') {
+			$v= '';
+		    }
+		} else {
+		    $v = '';
 		}
 		push(@archvalues,$v) unless !$v || $archadded{$v}++;
 	    } elsif (m/^(Package|Essential|Pre-Depends|Depends|Provides)$/ ||
@@ -176,10 +208,11 @@ for $_ (keys %fi) {
 	    }
 	}
     } elsif (s/^L //) {
-#print STDERR "L key >$_< value >$v<\n";
-        if (m/^Source$/) {
+        if (m/^Source$/i) {
             &setsourcepackage;
-        } elsif (m/^(Version|Maintainer|Changes|Urgency|Distribution|Date|Closes)$/) {
+        } elsif (m/^Maintainer$/i) {
+	    $f{"Changed-By"}=$v;
+        } elsif (m/^(Version|Changes|Urgency|Distribution|Date|Closes)$/i) {
             $f{$_}= $v;
         } elsif (s/^X[BS]*C[BS]*-//i) {
             $f{$_}= $v;
@@ -220,9 +253,15 @@ for $p (keys %p2f) {
                                  " file but $f2pri{$f} in files list");
 }
 
+# Extract version and origversion so we can add them to our fixed list
+# of substvars
+
+$version= $f{'Version'};
+$origversion= $version; $origversion =~ s/-[^-]+$//;
+$substvar{"dpkg:UpstreamVersion"}=$origversion;
+$substvar{"dpkg:Version"}=$version;
+
 if (!$binaryonly) {
-    $version= $f{'Version'};
-    $origversion= $version; $origversion =~ s/-[^-]+$//;
     $sec= $sourcedefault{'Section'};
     if (!length($sec)) { $sec='-'; &warn("missing Section for source files"); }
     $pri= $sourcedefault{'Priority'};
@@ -294,6 +333,7 @@ for $f (@sourcefiles,@fileslistfiles) {
 $f{'Source'}= $sourcepackage;
 
 $f{'Maintainer'}= $forcemaint if length($forcemaint);
+$f{'Changed-By'}= $forcechangedby if length($forcechangedby);
 
 for $f (qw(Version Distribution Maintainer Changes)) {
     defined($f{$f}) || &error("missing information for critical output field $f");
