@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 
-$dpkglibdir= ".";
+$dpkglibdir= "."; # This line modified by Makefile
 $version= '1.3.0'; # This line modified by Makefile
 
 $controlfile= 'debian/control';
@@ -17,9 +17,11 @@ require 'controllib.pl';
 
 sub usageversion {
     print STDERR
-"Debian GNU/Linux dpkg-gencontrol $version.  Copyright (C) 1996
-Ian Jackson.  This is free software; see the GNU General Public Licence
-version 2 or later for copying conditions.  There is NO warranty.
+"Debian dpkg-gencontrol $version. 
+Copyright (C) 1996 Ian Jackson.
+Copyright (C) 2000 Wichert Akkerman.
+This is free software; see the GNU General Public Licence version 2 or later
+for copying conditions.  There is NO warranty.
 
 Usage: dpkg-gencontrol [options ...]
 
@@ -30,6 +32,7 @@ Options:  -p<package>            print control file for package
           -v<forceversion>       set version of binary package
           -f<fileslistfile>      write files here instead of debian/files
           -P<packagebuilddir>    temporary build dir instead of debian/tmp
+	  -n<filename>           assume the package filename will be <filename>
           -O                     write to stdout, not .../DEBIAN/control
           -is                    include section field
           -ip                    include priority field
@@ -45,14 +48,16 @@ Options:  -p<package>            print control file for package
 $i=100;grep($fieldimps{$_}=$i--,
           qw(Package Version Section Priority Architecture Essential
              Pre-Depends Depends Recommends Suggests Enhances Optional 
-	     Conflicts Replaces Provides Installed-Size Maintainer Source
-	     Description Build-Depends Build-Depends-Indep Build-Conflicts
-	     Build-Conflicts-Indep Source));
+	     Conflicts Replaces Provides Installed-Size Origin Maintainer
+	     Bugs Source Description Build-Depends Build-Depends-Indep
+	     Build-Conflicts Build-Conflicts-Indep ));
 
 while (@ARGV) {
     $_=shift(@ARGV);
     if (m/^-p([-+0-9a-z.]+)$/) {
         $oppackage= $1;
+    } elsif (m/^-p([-+0-9a-zA-Z.]+)$/) {
+        &usageerr("Illegal package name \`$1'");
     } elsif (m/^-c/) {
         $controlfile= $';
     } elsif (m/^-l/) {
@@ -82,6 +87,8 @@ while (@ARGV) {
         $substvar{$1}= $';
     } elsif (m/^-T/) {
         $varlistfile= $';
+    } elsif (m/^-n/) {
+        $forcefilename= $';
     } elsif (m/^-h$/) {
         &usageversion; exit(0);
     } else {
@@ -109,10 +116,10 @@ for $_ (keys %fi) {
     $v= $fi{$_};
     if (s/^C //) {
 #print STDERR "G key >$_< value >$v<\n";
-        if (m/^Maintainer$/) { $f{$_}=$v; }
+        if (m/^Origin|Bugs|Maintainer$/) { $f{$_}=$v; }
         elsif (m/^Source$/) { &setsourcepackage; }
         elsif (s/^X[CS]*B[CS]*-//i) { $f{$_}= $v; }
-	elsif (m/^X[CS]+-|^Standards-Version$|^Build-(Depends|Conflicts)(-Indep)?$/i) { }
+	elsif (m/^X[CS]+-|^(Standards-Version|Uploaders)$|^Build-(Depends|Conflicts)(-Indep)?$/i) { }
 	elsif (m/^Section$|^Priority$/) { $spdefault{$_}= $v; }
         else { &unknown('general section of control info file'); }
     } elsif (s/^C$myindex //) {
@@ -160,6 +167,10 @@ for $_ (keys %fi) {
 }
 
 $f{'Version'}= $forceversion if length($forceversion);
+$version= $f{'Version'};
+$origversion= $version; $origversion =~ s/-[^-]+$//;
+$substvar{"dpkg:UpstreamVersion"}=$origversion;
+$substvar{"dpkg:Version"}=$version;
 
 for $f (qw(Section Priority)) {
     $spvalue{$f}= $spdefault{$f} unless length($spvalue{$f});
@@ -214,8 +225,9 @@ if (open(X,"< $fileslistfile")) {
 }
 $sversion=$f{'Version'};
 $sversion =~ s/^\d+://;
-print(Y &substvars(sprintf("%s_%s_%s.deb %s %s\n",
-                           $oppackage,$sversion,$f{'Architecture'},
+$forcefilename=sprintf("%s_%s_%s.deb", $oppackage,$sversion,$f{'Architecture'})
+	   unless ($forcefilename);
+print(Y &substvars(sprintf("%s %s %s\n", $forcefilename, 
                            &spfileslistvalue('Section'), &spfileslistvalue('Priority'))))
     || &syserr("write new entry to new files list file");
 close(Y) || &syserr("close new files list file");
