@@ -1,4 +1,7 @@
 #! /usr/bin/perl
+#
+# dpkg-shlibdeps
+# $Id: dpkg-shlibdeps.pl,v 1.16.4.2 2001/06/20 01:01:36 doogie Exp $
 
 $dpkglibdir="/usr/lib/dpkg";
 $version="1.4.1.19"; # This line modified by Makefile
@@ -22,9 +25,11 @@ require 'controllib.pl';
 
 sub usageversion {
     print STDERR
-"Debian GNU/Linux dpkg-shlibdeps $version.  Copyright (C) 1996
-Ian Jackson.  This is free software; see the GNU General Public Licence
-version 2 or later for copying conditions.  There is NO warranty.
+"Debian dpkg-shlibdeps $version.
+Copyright (C) 1996 Ian Jackson.
+Copyright (C) 2000 Wichert Akkerman.
+This is free software; see the GNU General Public Licence version 2 or
+later for copying conditions.  There is NO warranty.
 
 Usage:
   dpkg-shlibdeps [<option> ...] <executable>|-e<executable> [<option>] ...
@@ -88,62 +93,123 @@ sub isbin {
 
 for ($i=0;$i<=$#exec;$i++) {
     if (!isbin ($exec[$i])) { next; }
+    
+    # First we get an ldd output to see what libs + paths we have at out
+    # disposal.
+    my %so2path = ();
     defined($c= open(P,"-|")) || syserr("cannot fork for ldd");
     if (!$c) { exec("ldd","--",$exec[$i]); syserr("cannot exec ldd"); }
-    $nthisldd=0;
     while (<P>) {
-    	chomp;
-	if (m,^\s+(\S+)\s+\=\>\s+\1$,) {
-	    # shared libraries depend on themselves (unsure why)
-	    # Only under old ld.so
-	    $nthisldd++;
-	} elsif (m,\s+statically linked(\s+\(ELF\))?$,) {
-	    $nthisldd++;
-	} elsif (m,^\s+(\S+)\.so\.(\S+)\s+=>\s+(/\S+)(\s+\(0x.+\))?$,) {
-	    push(@libname,$1); push(@libsoname,$2); push(@libpath,$3);
-	    push(@libf,$execf[$i]);
-	    push(@libpaths,$3) if !$libpathadded{$3}++;
-	    $nthisldd++;
-	} elsif (m,^\s+(\S+)\.so\.(\S+)\s+=> not found,) {
-	    push(@libname,$1); push(@libsoname,$2); push(@libpath,$3);
-	    push(@libf,$execf[$i]);
-	    push(@libpaths,$3) if !$libpathadded{$3}++;
-	    $nthisldd++;
-	} else {
-	    &warn("unknown output from ldd on \`$exec[$i]': \`$_'");
+	if (m,^\s+(\S+)\s+=>\s+(\S+)\s+\(0x.+\)?$,) {
+	    $so2path{$1} = $2;
 	}
     }
     close(P); $? && subprocerr("ldd on \`$exec[$i]'");
-    $nthisldd || &warn("ldd on \`$exec[$i]' gave nothing on standard output");
+
+    # Now we get the direct deps of the program. We then check back with
+    # the ldd output from above to see what our path is.
+    defined($c= open(P,"-|")) || syserr("cannot fork for objdump");
+    if (!$c) { exec("objdump","-p","--",$exec[$i]); syserr("cannot exec objdump"); }
+    while (<P>) {
+	chomp;
+	if (m,^\s*NEEDED\s+,) {
+	    if (m,^\s*NEEDED\s+((\S+)\.so\.(\S+))$,) {
+		push(@libname,$2); push(@libsoname,$3);
+		push(@libf,$execf[$i]);
+		&warn("could not find path for $1") unless defined($so2path{$1});
+		push(@libfiles,$so2path{$1});
+	    } elsif (m,^\s*NEEDED\s+((\S+)-(\S+)\.so)$,) {
+		push(@libname,$2); push(@libsoname,$3);
+		push(@libf,$execf[$i]);
+		&warn("could not find path for $1") unless defined($so2path{$1});
+		push(@libfiles,$so2path{$1});
+	    } else {
+		m,^\s*NEEDED\s+(\S+)$,;
+		&warn("format of $1 not recognized");
+	    }
+	}
+    }
+    close(P); $? && subprocerr("objdump on \`$exec[$i]'");
 }
 
-if ($#libpaths >= 0) {
-    grep(s/\[\?\*/\\$&/g, @libpaths);
+# Now: See if it is in this package.  See if it is in any other package.
+sub searchdir {
+    my $dir = shift;
+    if(opendir(DIR, $dir)) {
+	my @dirents = readdir(DIR);
+	closedir(DIR);
+	for (@dirents) {
+	    if ( -f "$dir/$_/DEBIAN/shlibs" ) {
+		push(@curshlibs, "$dir/$_/DEBIAN/shlibs");
+		next;
+	    } elsif ( $_ !~ /^\./ && -d "$dir/$_" && ! -l "$dir/$_" ) {
+		&searchdir("$dir/$_");
+	    }
+	}
+    }
+}
+
+$searchdir = $exec[0];
+$curpackdir = "debian/tmp";
+do { $searchdir =~ s,/[^/]*$,,; } while($searchdir =~ m,/, && ! -d "$searchdir/DEBIAN");
+if ($searchdir =~ m,/,) {
+    $curpackdir = $searchdir;
+    $searchdir =~ s,/[^/]*,,;
+    &searchdir($searchdir);
+}
+
+if (1 || $#curshlibs >= 0) {
+    PRELIB: for ($i=0;$i<=$#libname;$i++) {
+	if(scanshlibsfile($shlibslocal,$libname[$i],$libsoname[$i],$libf[$i])
+	    || scanshlibsfile($shlibsoverride,$libname[$i],$libsoname[$i],$libf[$i])) {
+	    splice(@libname, $i, 1);
+	    splice(@libsoname, $i, 1);
+	    splice(@libf, $i, 1);
+	    splice(@libfiles, $i, 1);
+	    $i--;
+	    next PRELIB;
+	}
+	for my $shlibsfile (@curshlibs) {
+	    if(scanshlibsfile($shlibsfile, $libname[$i], $libsoname[$i], $libf[$i])) {
+		splice(@libname, $i, 1);
+		splice(@libsoname, $i, 1);
+		splice(@libf, $i, 1);
+		splice(@libfiles, $i, 1);
+		$i--;
+		next PRELIB;
+	    }
+	}
+    }
+}
+
+if ($#libfiles >= 0) {
+    grep(s/\[\?\*/\\$&/g, @libname);
     defined($c= open(P,"-|")) || syserr("cannot fork for dpkg --search");
-    if (!$c) { exec("dpkg","--search","--",@libpaths); syserr("cannot exec dpkg"); }
+    if (!$c) {
+        close STDERR; # we don't need to see dpkg's errors
+	open STDERR, "> /dev/null";
+        exec("dpkg","--search","--",map {"$_"} @libfiles); syserr("cannot exec dpkg");
+    }
     while (<P>) {
-       s/\n$//;
+       chomp;
        if (m/^local diversion |^diversion by/) {
            &warn("diversions involved - output may be incorrect");
            print(STDERR " $_\n") || syserr("write diversion info to stderr");
-       } elsif (m=^(\S+(, \S+)*): (/.+)$=) {
-           $pathpackages{$+}= $1;
+       } elsif (m=^(\S+(, \S+)*): (\S+)$=) {
+           push @{$pathpackages{$+}}, split(/, /, $1);
        } else {
            &warn("unknown output from dpkg --search: \`$_'");
        }
     }
-    close(P); $? && subprocerr("dpkg --search");
+    close(P); 
 }
 
 LIB: for ($i=0;$i<=$#libname;$i++) {
-    scanshlibsfile($shlibslocal,$libname[$i],$libsoname[$i],$libf[$i]) && next;
-    scanshlibsfile($shlibsoverride,$libname[$i],$libsoname[$i],$libf[$i]) && next;
-    if (!defined($pathpackages{$libpath[$i]})) {
-        &warn("could not find any packages for $libpath[$i]".
+    if (!defined($pathpackages{$libfiles[$i]})) {
+        &warn("could not find any packages for $libfiles[$i]".
               " ($libname[$i].so.$libsoname[$i])");
     } else {
-        @packages= split(/, /,$pathpackages{$libpath[$i]});
-        for $p (@packages) {
+        for $p (@{$pathpackages{$libfiles[$i]}}) {
             scanshlibsfile("$shlibsppdir/$p$shlibsppext",
                            $libname[$i],$libsoname[$i],$libf[$i])
                 && next LIB;
@@ -151,7 +217,7 @@ LIB: for ($i=0;$i<=$#libname;$i++) {
     }
     scanshlibsfile($shlibsdefault,$libname[$i],$libsoname[$i],$libf[$i]) && next;
     &warn("unable to find dependency information for ".
-          "shared library $libname[$i] (soname $libsoname[$i], path $libpath[$i], ".
+          "shared library $libname[$i] (soname $libsoname[$i], path $libfiles[$i], ".
           "dependency field $libf[$i])");
 }
 
@@ -161,10 +227,9 @@ sub scanshlibsfile {
     $fn= "./$fn" if $fn =~ m/^\s/;
     if (!open(SLF,"< $fn")) {
         $! == ENOENT || syserr("unable to open shared libs info file \`$fn'");
-#print STDERR "scanshlibsfile($fn,$ln,$lsn,$lf) ... ENOENT\n";
         return 0;
     }
-#print STDERR "scanshlibsfile($fn,$ln,$lsn,$lf) ...\n";
+
     while (<SLF>) {
         s/\s*\n$//; next if m/^\#/;
         if (!m/^\s*(\S+)\s+(\S+)/) {
@@ -172,6 +237,7 @@ sub scanshlibsfile {
             next;
         }
         next if $1 ne $ln || $2 ne $lsn;
+        return 1 if $fn eq "$curpackdir/DEBIAN/shlibs";
         $da= $';
         for $dv (split(/,/,$da)) {
             $dv =~ s/^\s+//; $dv =~ s/\s+$//;
@@ -196,7 +262,6 @@ sub scanshlibsfile {
 }
 
 if (!$stdout) {
-    $varlistfile="./$varlistfile" if $fileslistfile =~ m/^\s/;
     open(Y,"> $varlistfile.new") ||
         syserr("open new substvars file \`$varlistfile.new'");
     chown(@fowner, "$varlistfile.new") ||
