@@ -8,7 +8,7 @@ my %dirincluded;
 my %notfileobject;
 my $fn;
 
-$diff_ignore_default_regexp = '^.*~$|DEAD_JOE|(?:/CVS|/RCS|/.deps)(?:$|/.*$)';
+$diff_ignore_default_regexp = '^.*~$|^\..*\.swp|DEADJOE|(?:/CVS|/RCS|/.deps)(?:$|/.*$)';
 
 $sourcestyle = 'X';
 $dscformat = "1.0";
@@ -26,7 +26,7 @@ delete $ENV{'POSIXLY_CORRECT'};
 
 sub usageversion {
     print STDERR
-"Debian GNU/Linux dpkg-source $version.  Copyright (C) 1996
+"Debian dpkg-source $version.  Copyright (C) 1996
 Ian Jackson and Klee Dienes.  This is free software; see the GNU
 General Public Licence version 2 or later for copying conditions.
 There is NO warranty.
@@ -40,6 +40,8 @@ Build options:   -c<controlfile>     get control info from this file
                  -T<varlistfile>     read variables here, not debian/substvars
                  -D<field>=<value>   override or add a .dsc field and value
                  -U<field>           remove a field
+                 -W                  Turn certain errors into warnings. 
+                 -E                  When -W is enabled, -E disables it.
                  -sa                 auto select orig source (-sA is default)
                  -i[<regexp>]        filter out files to ignore diffs of.
                                      Defaults to: '$diff_ignore_default_regexp'
@@ -57,9 +59,20 @@ General options: -h                  print this message
 ";
 }
 
+sub handleformat {
+	local $fmt	= shift;
+	local $ourfmt	= $dscformat;
+
+	$fmt =~ s/(.*)\.\d*/$1/;
+	$ourfmt =~ s/(.*)\.\d*/$1/;
+
+	return ($fmt==$ourfmt);
+}
+
+
 $i = 100;
 grep ($fieldimps {$_} = $i--,
-      qw (Format Source Version Binary Maintainer Architecture
+      qw (Format Source Version Binary Origin Maintainer Architecture
       Standards-Version Build-Depends Build-Depends-Indep Build-Conflicts
       Build-Conflicts-Indep));
 
@@ -89,6 +102,10 @@ while (@ARGV && $ARGV[0] =~ m/^-/) {
         $varlistfile= $';
     } elsif (m/^-h$/) {
         &usageversion; exit(0);
+    } elsif (m/^-W$/) {
+        $warnable_error= 1;
+    } elsif (m/^-E$/) {
+        $warnable_error= 0;
     } elsif (m/^--$/) {
         last;
     } else {
@@ -98,6 +115,8 @@ while (@ARGV && $ARGV[0] =~ m/^-/) {
 
 defined($opmode) || &usageerr("need -x or -b");
 
+$SIG{'PIPE'} = 'DEFAULT';
+
 if ($opmode eq 'build') {
 
     $sourcestyle =~ y/X/A/;
@@ -122,19 +141,16 @@ if ($opmode eq 'build') {
     for $_ (keys %fi) {
         $v= $fi{$_};
         if (s/^C //) {
-#print STDERR "G key >$_< value >$v<\n";
             if (m/^Source$/) { &setsourcepackage; }
-            elsif (m/^Standards-Version$|^Maintainer$/) { $f{$_}= $v; }
+            elsif (m/^(Standards-Version|Origin|Maintainer|Uploaders)$/) { $f{$_}= $v; }
 	    elsif (m/^Build-(Depends|Conflicts)(-Indep)?$/i) { $f{$_}= $v; }
             elsif (s/^X[BC]*S[BC]*-//i) { $f{$_}= $v; }
-            elsif (m/^(Section|Priority|Files)$/ || m/^X[BC]+-/i) { }
+            elsif (m/^(Section|Priority|Files|Bugs)$/ || m/^X[BC]+-/i) { }
             else { &unknown('general section of control info file'); }
         } elsif (s/^C(\d+) //) {
-#print STDERR "P key >$_< value >$v<\n";
             $i=$1; $p=$fi{"C$i Package"};
             push(@binarypackages,$p) unless $packageadded{$p}++;
             if (m/^Architecture$/) {
-#print STDERR "$p >$v< >".join(' ',@sourcearch)."<\n";
                 if ($v eq 'any') {
                     @sourcearch= ('any');
                 } elsif ($v eq 'all') {
@@ -144,16 +160,16 @@ if ($opmode eq 'build') {
                         @sourcearch= ('any');
                     }
                 } else {
-                    if (grep($sourcearch[0] eq $_, 'any','all')) {
-                        @sourcearch= ('any');
-                    } else {
+		    if (grep($sourcearch[0] eq $_, 'any','all'))  {
+			@sourcearch= ('any');
+		    } else {
                         for $a (split(/\s+/,$v)) {
                             &error("architecture $a only allowed on its own".
                                    " (list for package $p is \`$a')")
                                    if grep($a eq $_, 'any','all');
                             push(@sourcearch,$a) unless $archadded{$a}++;
                         }
-                    }
+		    }
                 }
                 $f{'Architecture'}= join(' ',@sourcearch);
             } elsif (s/^X[BC]*S[BC]*-//i) {
@@ -166,7 +182,6 @@ if ($opmode eq 'build') {
                 &unknown("package's section of control info file");
             }
         } elsif (s/^L //) {
-#print STDERR "L key >$_< value >$v<\n";
             if (m/^Source$/) {
                 &setsourcepackage;
             } elsif (m/^Version$/) {
@@ -201,9 +216,7 @@ if ($opmode eq 'build') {
     $basenamerev= $sourcepackage.'_'.$version;
     $basename= $sourcepackage.'_'.$upstreamversion;
     $basedirname= $basename;
-#print STDERR ">$basedirname<\n";
     $basedirname =~ s/_/-/;
-#print STDERR ">$basedirname<\n";
 
     $origdir= "$dir.orig";
     $origtargz= "$basename.orig.tar.gz";
@@ -272,8 +285,6 @@ if ($opmode eq 'build') {
         $tarname= "$basenamerev.tar.gz";
     }
 
-#print STDERR ">$dir|$origdir|$origtargz|$sourcestyle<\n";
-
     if ($sourcestyle =~ m/[nurUR]/) {
 
         if (stat($tarname)) {
@@ -284,15 +295,12 @@ if ($opmode eq 'build') {
             &syserr("unable to check for existence of \`$tarname'");
         }
 
-#print STDERR ">$tarname|$tardirbase|$tardirname<\n";
-    
         print("$progname: building $sourcepackage in $tarname\n")
             || &syserr("write building tar message");
         &forkgzipwrite("$tarname.new");
         defined($c2= fork) || &syserr("fork for tar");
         if (!$c2) {
             chdir($tardirbase) || &syserr("chdir to above (orig) source $tardirbase");
-#system('pwd && ls');
             open(STDOUT,">&GZIP") || &syserr("reopen gzip for tar");
             # FIXME: put `--' argument back when tar is fixed
             exec('tar','-cf','-',$tardirname); &syserr("exec tar");
@@ -325,7 +333,9 @@ if ($opmode eq 'build') {
         }
 
         $expectprefix= $origdir; $expectprefix =~ s,^\./,,;
-        checktarsane($origtargz,$expectprefix);
+# tar checking is disabled, there are too many broken tar archives out there
+# which we can still handle anyway.
+#        checktarsane($origtargz,$expectprefix);
         mkdir("$origtargz.tmp-nest",0755) ||
             &syserr("unable to create \`$origtargz.tmp-nest'");
         extracttar($origtargz,"$origtargz.tmp-nest",$expectprefix);
@@ -377,6 +387,7 @@ if ($opmode eq 'build') {
                 }
                 defined($c3= open(DIFFGEN,"-|")) || &syserr("fork for diff");
                 if (!$c3) {
+		    $ENV{'LC_ALL'}= 'C';
 		    $ENV{'LANG'}= 'C';
                     exec('diff','-u',
                          '-L',"$basedirname.orig/$fn",
@@ -490,10 +501,12 @@ if ($opmode eq 'build') {
     }
 
     if (defined $fi{'S Format'}) {
-        $dscformat=$fi{'S Format'};
-	$dscformat != "1.0" &&
+	if (not handleformat($fi{'S Format'})) {
 	    &error("Unsupported format of .dsc file ($dscformat)");
+        }
+        $dscformat=$fi{'S Format'};
     }
+
     $sourcepackage =~ m/[^.0-9]/ &&
         &error("dsc format contains illegal character \`$&'");
 
@@ -530,7 +543,9 @@ if ($opmode eq 'build') {
     checkstats($tarfile);
     checkstats($difffile) if length($difffile);
 
-    checktarsane("$dscdir/$tarfile",$expectprefix);
+# tar checking is disabled, there are too many broken tar archives out there
+# which we can still handle anyway.
+#    checktarsane("$dscdir/$tarfile",$expectprefix);
 
     if (length($difffile)) {
             
@@ -657,6 +672,7 @@ if ($opmode eq 'build') {
         if (!$c2) {
             open(STDIN,"<&GZIP") || &syserr("reopen gzip for patch");
             chdir($newdirectory) || &syserr("chdir to $newdirectory for patch");
+	    $ENV{'LC_ALL'}= 'C';
 	    $ENV{'LANG'}= 'C';
             exec('patch','-s','-t','-F','0','-N','-p1','-u',
                  '-V','never','-g0','-b','-z','.dpkg-orig');
@@ -689,8 +705,7 @@ if ($opmode eq 'build') {
     $plainmode= $execmode & ~0111;
     $fifomode= ($plainmode & 0222) | (($plainmode & 0222) << 1);
     for $fn (@filesinarchive) {
-        $fn= substr($fn,length($expectprefix)+1);
-        $fn= "$newdirectory/$fn";
+	$fn=~ s,^$expectprefix,$newdirectory,;
         (@s= lstat($fn)) || &syserr("cannot stat extracted object \`$fn'");
         $mode= $s[2];
         if (-d _) {
@@ -719,6 +734,7 @@ sub checkstats {
     (@s= stat(STDIN)) || &syserr("cannot fstat $dscdir/$f");
     $s[7] == $size{$f} || &error("file $f has size $s[7] instead of expected $size{$f}");
     $m= `md5sum`; $? && subprocerr("md5sum $f"); $m =~ s/\n$//;
+    $m =~ s/ *-$//; # Remove trailing spaces and -, to work with GNU md5sum
     $m =~ m/^[0-9a-f]{32}$/ || &failure("md5sum of $f gave bad output \`$m'");
     $m eq $md5sum{$f} || &error("file $f has md5sum $m instead of expected $md5sum{$f}");
     open(STDIN,"</dev/null") || &syserr("reopen stdin from /dev/null");
@@ -752,6 +768,8 @@ sub checktarcpio {
     &forkgzipread ("$tarfileread");
     if (! defined ($c2 = open (CPIO,"-|"))) { &syserr ("fork for cpio"); }
     if (!$c2) {
+	$ENV{'LC_ALL'}= 'C';
+	$ENV{'LANG'}= 'C';
         open (STDIN,"<&GZIP") || &syserr ("reopen gzip for cpio");
         &cpiostderr;
         exec ('cpio','-0t');
@@ -760,7 +778,6 @@ sub checktarcpio {
     close (GZIP);
 
     $/ = "\0";
-    open (CPIO, "<cpiolog");
     while (defined ($fn = <CPIO>)) {
 
         $fn =~ s/\0$//;
@@ -769,13 +786,20 @@ sub checktarcpio {
 	my $pname = $fn;
 	$pname =~ y/ -~/?/c;
 
+        if ($fn =~ m/\n/) {
+	    &error ("tarfile \`$tarfileread' contains object with".
+		    " newline in its name ($pname)");
+	}
+
+	next if ($fn eq '././@LongLink');
+
 	if (! $tarprefix) {
 	    if ($fn =~ m/\n/) {
                 &error("first output from cpio -0t (from \`$tarfileread') ".
                        "contains newline - you probably have an out of ".
                        "date version of cpio.  GNU cpio 2.4.2-2 is known to work");
 	    }
-	    $tarprefix = ($fn =~ m,([^/]*)[/],)[0];
+	    $tarprefix = ($fn =~ m,((\./)*[^/]*)[/],)[0];
 	    # need to check for multiple dots on some operating systems
 	    # empty tarprefix (due to regex failer) will match emptry string
 	    if ($tarprefix =~ /^[.]*$/) {
@@ -784,13 +808,6 @@ sub checktarcpio {
 	    }
 	}
 
-        if ($fn =~ m/\n/) {
-	    &error ("tarfile \`$tarfileread' contains object with".
-		    " newline in its name ($pname)");
-	}
-
-	next if ($fn eq '././@LongLink');
-
 	my $fprefix = substr ($fn, 0, length ($tarprefix));
         my $slash = substr ($fn, length ($tarprefix), 1);
         if ((($slash ne '/') && ($slash ne '')) || ($fprefix ne $tarprefix)) {
@@ -811,7 +828,6 @@ sub checktarcpio {
     $/= "\n";
 
     my $tarsubst = quotemeta ($tarprefix);
-    @filesinarchive = map { s/^$tarsubst/$wpfx/; $_ } @filesinarchive;
 
     return $tarprefix;
 }
@@ -830,6 +846,7 @@ sub checktarsane {
     &forkgzipread ("$tarfileread");
     if (! defined ($c2 = open (TAR,"-|"))) { &syserr ("fork for tar -t"); }
     if (! $c2) {
+	$ENV{'LC_ALL'}= 'C';
         $ENV{'LANG'}= 'C';
         open (STDIN, "<&GZIP") || &syserr ("reopen gzip for tar -t");
         exec ('tar', '-vvtf', '-'); &syserr ("exec tar -vvtf -");
@@ -837,7 +854,6 @@ sub checktarsane {
     close (GZIP);
 
     my $efix= 0;
-    open (TAR, "<tarlog");
     while (<TAR>) {
 
         chomp;
@@ -853,15 +869,15 @@ sub checktarsane {
                    "unknown or forbidden type \`".substr($_,0,1)."'");
         my $type = $&;
 
-        if ($mode =~ /^l/) { $_ =~ s/ -\> .*//; }
+        if ($mode =~ /^l/) { $_ =~ s/ -> .*//; }
         s/ link to .+//;
 
-	if (length ($_) <= 48) { 
+	my @tarfields = split(' ', $_, 6);
+	if (@tarfields < 6) { 
 	    &error ("tarfile \`$tarfileread' contains incomplete entry \`$_'\n");
 	}
 
-	my $tarfn = substr ($_, 48, length ($_) - 48);
-	$tarfn = deoctify ($tarfn);
+	my $tarfn = deoctify ($tarfields[5]);
 
 	# store printable name of file for error messages
 	my $pname = $tarfn;
@@ -870,6 +886,12 @@ sub checktarsane {
 	# fetch name of file as given by cpio
         $fn = $filesinarchive[$efix++];
 
+	my $l = length($fn);
+	if (substr ($tarfn, 0, $l + 4) eq "$fn -> ") {
+		# This is a symlink, as listed by tar.  cpio doesn't
+		# give us the targets of the symlinks, so we ignore this.
+		$tarfn = substr($tarfn, 0, $l);
+	}
 	if ($tarfn ne $fn) {
 	    if ((length ($fn) == 99) && (length ($tarfn) >= 99)
 		&& (substr ($fn, 0, 99) eq substr ($tarfn, 0, 99))) {
@@ -877,6 +899,10 @@ sub checktarsane {
 		# to the first 100 characters.  let it slide for now.
 		&warn ("filename \`$pname' was truncated by cpio;" .
 		       " unable to check full pathname");
+		# Since it didn't match, later checks will not be able
+		# to stat this file, so we replace it with the filename
+		# fetched from tar.
+		$filesinarchive[$efix-1] = $tarfn;
 	    } else {
 		&error ("tarfile \`$tarfileread' contains unexpected object".
 			" listed by tar as \`$_'; expected \`$pname'");
@@ -906,20 +932,23 @@ sub checktarsane {
 	}
 
         if ($type eq 'd') { $tarfn =~ s,/$,,; }
+        $tarfn =~ s,(\./)*,,;
         my $dirname = $tarfn;
 
         if (($dirname =~ s,/[^/]+$,,) && (! defined ($dirincluded{$dirname}))) {
-            &error ("tarfile \`$tarfileread' contains object \`$pname' but its containing ".
+            &warnerror ("tarfile \`$tarfileread' contains object \`$pname' but its containing ".
 		    "directory \`$dirname' does not precede it");
+	    $dirincluded{$dirname} = 1;
 	}
 	if ($type eq 'd') { $dirincluded{$tarfn} = 1; }
         if ($type ne '-') { $notfileobject{$tarfn} = 1; }
     }
     close (TAR);
-    #$? && subprocerr ("tar -vvtf");
+    $? && subprocerr ("tar -vvtf");
     &reapgzip;
 
     my $tarsubst = quotemeta ($tarprefix);
+    @filesinarchive = map { s/^$tarsubst/$wpfx/; $_ } @filesinarchive;
     %dirincluded = map { s/^$tarsubst/$wpfx/; $_=>1 } (keys %dirincluded);
     %notfileobject = map { s/^$tarsubst/$wpfx/; $_=>1 } (keys %notfileobject);
 }
@@ -931,9 +960,9 @@ sub extracttar {
     &forkgzipread("$tarfileread");
     defined($c2= fork) || &syserr("fork for tar -xkf -");
     if (!$c2) {
-        chdir("$dirchdir") || &syserr("cannot chdir to \`$dirchdir' for tar extract");
-        open(STDIN,"<&GZIP") || &syserr("reopen gzip for cpio -i");
+        open(STDIN,"<&GZIP") || &syserr("reopen gzip for tar -xkf -");
         &cpiostderr;
+        chdir("$dirchdir") || &syserr("cannot chdir to \`$dirchdir' for tar extract");
         exec('tar','-xkf','-'); &syserr("exec tar -xkf -");
     }
     close(GZIP);
@@ -961,7 +990,7 @@ sub extracttar {
 
 sub cpiostderr {
     open(STDERR,"| egrep -v '^[0-9]+ blocks\$' >&2") ||
-        &syserr("reopen stderr for cpio to grep out blocks message");
+        &syserr("reopen stderr for tar to grep out blocks message");
 }
 
 sub setfile {
@@ -1003,7 +1032,6 @@ sub unrepdiff2 {
 }
 
 sub forkgzipwrite {
-#print STDERR "forkgzipwrite $_[0]\n";
     open(GZIPFILE,"> $_[0]") || &syserr("create file $_[0]");
     pipe(GZIPREAD,GZIP) || &syserr("pipe for gzip");
     defined($cgz= fork) || &syserr("fork for gzip");
@@ -1017,7 +1045,7 @@ sub forkgzipwrite {
 }
 
 sub forkgzipread {
-#print STDERR "forkgzipread $_[0]\n";
+    local $SIG{PIPE} = 'DEFAULT';
     open(GZIPFILE,"< $_[0]") || &syserr("read file $_[0]");
     pipe(GZIP,GZIPWRITE) || &syserr("pipe for gunzip");
     defined($cgz= fork) || &syserr("fork for gunzip");
@@ -1031,7 +1059,6 @@ sub forkgzipread {
 }
 
 sub reapgzip {
-#print STDERR "reapgzip $_[0]\n";
     $cgz == waitpid($cgz,0) || &syserr("wait for gzip");
     !$? || ($gzipsigpipeok && WIFSIGNALED($?) && WTERMSIG($?)==SIGPIPE) ||
         subprocerr("gzip");
