@@ -1,8 +1,8 @@
 #!/usr/bin/perl --
 
-#use POSIX; &ENOENT;
-sub ENOENT { 2; }
-# Sorry about this, but the errno-part of POSIX.pm isn't in perl-*-base
+$admindir= "/var/lib/dpkg"; # This line modified by Makefile
+$dpkglibdir= "../utils"; # This line modified by Makefile
+$version= '0.93.80'; # This line modified by Makefile
 
 # Global variables:
 #  $alink            Alternative we are managing (ie the symlink we're making/removing) (install only)
@@ -15,7 +15,7 @@ sub ENOENT { 2; }
 #                       expected: alternative with highest priority is the active alternative
 #                       expected-inprogress: busy selecting alternative with highest priority
 #                       unexpected: alternative another alternative is active / error during readlink
-#                       nonexistant: alternative-symlink does not exist
+#                       nonexistent: alternative-symlink does not exist
 #  $link             Link we are working with
 #  @slavenames       List with names of slavelinks
 #  %slavenum         Map from name of slavelink to slave-index (into @slavelinks)
@@ -25,11 +25,16 @@ sub ENOENT { 2; }
 #  %priorities       Map from @version-index to priority
 #  %slavepath        Map from (@version-index,slavename) to slave-path
 
-$version= '0.93.80'; # This line modified by Makefile
+$enoent=`$dpkglibdir/enoent` || die "Cannot get ENOENT value from $dpkglibdir/enoent: $!";
+$enoent=`$dpkglibdir/enoent`;
+sub ENOENT { $enoent; }
+
 sub usageversion {
     print(STDERR <<END)
-Debian GNU/Linux update-alternatives $version.  Copyright (C) 1995
-Ian Jackson.  This is free software; see the GNU General Public Licence
+Debian update-alternatives $version.
+Copyright (C) 1995 Ian Jackson.
+Copyright (C) 2000,2001,2002 Wichert Akkerman
+This is free software; see the GNU General Public Licence
 version 2 or later for copying conditions.  There is NO warranty.
 
 Usage: update-alternatives --install <link> <name> <path> <priority>
@@ -52,7 +57,7 @@ sub quit { print STDERR "update-alternatives: @_\n"; exit(2); }
 sub badusage { print STDERR "update-alternatives: @_\n\n"; &usageversion; exit(2); }
 
 $altdir= '/etc/alternatives';
-$admindir= '/var/lib/dpkg/alternatives';
+$admindir= $admindir . '/alternatives';
 $testmode= 0;
 $verbosemode= 0;
 $mode='';
@@ -160,6 +165,7 @@ if (open(AF,"$admindir/$name")) {
 if ($mode eq 'display') {
     if (!$dataread) {
         &pr("No alternatives for $name.");
+	exit 1;
     } else {
         &pr("$name - status is $manual.");
         if (defined($linkname= readlink("$altdir/$name"))) {
@@ -234,10 +240,10 @@ if ($mode eq 'auto') {
     $state= 'nonexistent';
     $manual= 'auto';
 } elsif ($state eq 'nonexistent') {
-    if ($mode eq 'manual') {
+    if ($manual eq 'manual') {
         &pr("$altdir/$name has been deleted, returning to automatic selection.")
           if $verbosemode > 0;
-        $mode= 'auto';
+        $manual= 'auto';
     }
 }
 
