@@ -33,6 +33,7 @@
 #include <dpkg-db.h>
 #include "dpkg-split.h"
 
+void do_split(const char *const *argv) NONRETURNING;
 void do_split(const char *const *argv) {
   const char *sourcefile, *prefix;
   char *palloc;
@@ -66,9 +67,6 @@ void do_split(const char *const *argv) {
   m_dup2(fd,0);
   execl(MKSPLITSCRIPT,MKSPLITSCRIPT,
         sourcefile,partsizebuf,prefix,lengthbuf,partsizeallowbuf,msdos?"yes":"no",
-        (char*)0);
+        NULL);
   ohshite(_("unable to exec mksplit"));
 }
-
-
-
